"""Comprehensive validation tests for all 25 ANNCSU API operations.

This test suite validates response schemas for all operations defined in the
OpenAPI specification against actual API responses.
"""

from pathlib import Path
from unittest.mock import Mock

import httpx
import pytest

from anncsu.common.validation import ResponseValidator


class TestEsisteOdonimoOperations:
    """Tests for esisteodonimo operations (check if street name exists)."""

    @pytest.fixture
    def spec_path(self):
        """Fixture providing path to OpenAPI spec."""
        return (
            Path(__file__).parent.parent.parent
            / "oas"
            / "dev"
            / "Specifica API - ANNCSU – Consultazione per le PA.yaml"
        )

    @pytest.fixture
    def validator(self, spec_path):
        """Fixture providing ResponseValidator instance."""
        return ResponseValidator(spec_path)

    def test_esiste_odonimo_get_query_param_success(self, validator):
        """Test validation of esisteOdonimoGetQueryParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response.content = b'{"res": "esisteodonimo", "data": true}'
        response.json.return_value = {"res": "esisteodonimo", "data": True}

        is_valid, errors = validator.validate_response(
            response, "esisteOdonimoGetQueryParam"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_esiste_odonimo_post_success(self, validator):
        """Test validation of esisteOdonimoPost 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response.content = b'{"res": "esisteodonimo", "data": false}'
        response.json.return_value = {"res": "esisteodonimo", "data": False}

        is_valid, errors = validator.validate_response(response, "esisteOdonimoPost")

        assert is_valid is True
        assert len(errors) == 0

    def test_esiste_odonimo_get_path_param_success(self, validator):
        """Test validation of esisteOdonimoGetPathParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response.content = b'{"res": "esisteodonimo", "data": true}'
        response.json.return_value = {"res": "esisteodonimo", "data": True}

        is_valid, errors = validator.validate_response(
            response, "esisteOdonimoGetPathParam"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_esiste_odonimo_error_400(self, validator):
        """Test validation of esisteOdonimoGetQueryParam 400 error response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 400
        response.headers = {"content-type": "application/problem+json"}
        response.content = (
            b'{"title": "Bad Request", "detail": "Invalid codcom parameter"}'
        )
        response.json.return_value = {
            "title": "Bad Request",
            "detail": "Invalid codcom parameter",
        }

        is_valid, errors = validator.validate_response(
            response, "esisteOdonimoGetQueryParam"
        )

        assert is_valid is True
        assert len(errors) == 0


class TestEsisteAccessoOperations:
    """Tests for esisteaccesso operations (check if civic number exists)."""

    @pytest.fixture
    def spec_path(self):
        """Fixture providing path to OpenAPI spec."""
        return (
            Path(__file__).parent.parent.parent
            / "oas"
            / "dev"
            / "Specifica API - ANNCSU – Consultazione per le PA.yaml"
        )

    @pytest.fixture
    def validator(self, spec_path):
        """Fixture providing ResponseValidator instance."""
        return ResponseValidator(spec_path)

    def test_esiste_accesso_get_query_param_success(self, validator):
        """Test validation of esisteAccessoGetQueryParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response.content = b'{"res": "esisteaccesso", "data": true}'
        response.json.return_value = {"res": "esisteaccesso", "data": True}

        is_valid, errors = validator.validate_response(
            response, "esisteAccessoGetQueryParam"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_esiste_accesso_post_success(self, validator):
        """Test validation of esisteAccessoPost 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response.content = b'{"res": "esisteaccesso", "data": false}'
        response.json.return_value = {"res": "esisteaccesso", "data": False}

        is_valid, errors = validator.validate_response(response, "esisteAccessoPost")

        assert is_valid is True
        assert len(errors) == 0

    def test_esiste_accesso_get_path_param_success(self, validator):
        """Test validation of esisteAccessoGetPathParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response.content = b'{"res": "esisteaccesso", "data": true}'
        response.json.return_value = {"res": "esisteaccesso", "data": True}

        is_valid, errors = validator.validate_response(
            response, "esisteAccessoGetPathParam"
        )

        assert is_valid is True
        assert len(errors) == 0


class TestElencoOdonimiOperations:
    """Tests for elencoodonimi operations (list street names)."""

    @pytest.fixture
    def spec_path(self):
        """Fixture providing path to OpenAPI spec."""
        return (
            Path(__file__).parent.parent.parent
            / "oas"
            / "dev"
            / "Specifica API - ANNCSU – Consultazione per le PA.yaml"
        )

    @pytest.fixture
    def validator(self, spec_path):
        """Fixture providing ResponseValidator instance."""
        return ResponseValidator(spec_path)

    def test_elenco_odonimi_get_query_param_success(self, validator):
        """Test validation of elencoOdonimiGetQueryParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoodonimi",
            "data": [
                {
                    "dug": "VIA",
                    "denomuff": "ROMA",
                    "denomloc": None,
                    "denomlingua1": None,
                    "denomlingua2": None,
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "elencoOdonimiGetQueryParam"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_elenco_odonimi_post_success(self, validator):
        """Test validation of elencoOdonimiPost 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoodonimi",
            "data": [
                {
                    "dug": "AUTOSTRADA",
                    "denomuff": "ROMA AEROPORTO DI FIUMICINO",
                    "denomloc": "ROMA-FIUMICINO",
                    "denomlingua1": None,
                    "denomlingua2": None,
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(response, "elencoOdonimiPost")

        assert is_valid is True
        assert len(errors) == 0

    def test_elenco_odonimi_get_path_param_success(self, validator):
        """Test validation of elencoOdonimiGetPathParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoodonimi",
            "data": [
                {
                    "dug": "PIAZZA",
                    "denomuff": "SAN PIETRO",
                    "denomloc": None,
                    "denomlingua1": None,
                    "denomlingua2": None,
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "elencoOdonimiGetPathParam"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_elenco_odonimi_not_found_404(self, validator):
        """Test validation of elencoOdonimiGetQueryParam 404 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 404
        response.headers = {"content-type": "application/problem+json"}
        response.content = b'{"title": "Not Found", "detail": "No streets found"}'
        response.json.return_value = {
            "title": "Not Found",
            "detail": "No streets found",
        }

        is_valid, errors = validator.validate_response(
            response, "elencoOdonimiGetQueryParam"
        )

        assert is_valid is True
        assert len(errors) == 0


class TestElencoAccessiOperations:
    """Tests for elencoaccessi operations (list civic numbers)."""

    @pytest.fixture
    def spec_path(self):
        """Fixture providing path to OpenAPI spec."""
        return (
            Path(__file__).parent.parent.parent
            / "oas"
            / "dev"
            / "Specifica API - ANNCSU – Consultazione per le PA.yaml"
        )

    @pytest.fixture
    def validator(self, spec_path):
        """Fixture providing ResponseValidator instance."""
        return ResponseValidator(spec_path)

    def test_elenco_accessi_get_query_param_success(self, validator):
        """Test validation of elencoAccessiGetQueryParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoaccessi",
            "data": [
                {
                    "civico": "1",
                    "esp": "A",
                    "specif": "ROSSO",
                    "metrico": "1200",
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "elencoAccessiGetQueryParam"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_elenco_accessi_post_success(self, validator):
        """Test validation of elencoAccessiPost 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoaccessi",
            "data": [{"civico": "42", "esp": "", "specif": "", "metrico": "5400"}],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(response, "elencoAccessiPost")

        assert is_valid is True
        assert len(errors) == 0

    def test_elenco_accessi_get_path_param_success(self, validator):
        """Test validation of elencoAccessiGetPathParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoaccessi",
            "data": [{"civico": "10", "esp": "B", "specif": "BIS", "metrico": "2500"}],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "elencoAccessiGetPathParam"
        )

        assert is_valid is True
        assert len(errors) == 0


class TestElencoodonimiProgOperations:
    """Tests for elencoodonimiprog operations (list streets with national progressive)."""

    @pytest.fixture
    def spec_path(self):
        """Fixture providing path to OpenAPI spec."""
        return (
            Path(__file__).parent.parent.parent
            / "oas"
            / "dev"
            / "Specifica API - ANNCSU – Consultazione per le PA.yaml"
        )

    @pytest.fixture
    def validator(self, spec_path):
        """Fixture providing ResponseValidator instance."""
        return ResponseValidator(spec_path)

    def test_elencoodonimi_prog_get_query_param_success(self, validator):
        """Test validation of elencoodonimiprogGetQueryParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoodonimiprog",
            "data": [
                {
                    "prognaz": "919572",
                    "dug": "AUTOSTRADA",
                    "denomuff": "ROMA AEROPORTO DI FIUMICINO",
                    "denomloc": None,
                    "denomlingua1": None,
                    "denomlingua2": None,
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "elencoodonimiprogGetQueryParam"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_elencoodonimi_prog_post_success(self, validator):
        """Test validation of elencoodonimiprogPost 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoodonimiprog",
            "data": [
                {
                    "prognaz": "123456",
                    "dug": "VIA",
                    "denomuff": "GARIBALDI",
                    "denomloc": None,
                    "denomlingua1": None,
                    "denomlingua2": None,
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "elencoodonimiprogPost"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_elencoodonimi_prog_get_path_param_success(self, validator):
        """Test validation of elencoodonimiprogGetPathParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoodonimiprog",
            "data": [
                {
                    "prognaz": "789012",
                    "dug": "CORSO",
                    "denomuff": "VITTORIO EMANUELE",
                    "denomloc": "C. VITTORIO EMANUELE",
                    "denomlingua1": None,
                    "denomlingua2": None,
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "elencoodonimiprogGetPathParam"
        )

        assert is_valid is True
        assert len(errors) == 0


class TestElencoaccessiProgOperations:
    """Tests for elencoaccessiprog operations (list civic numbers with national progressive)."""

    @pytest.fixture
    def spec_path(self):
        """Fixture providing path to OpenAPI spec."""
        return (
            Path(__file__).parent.parent.parent
            / "oas"
            / "dev"
            / "Specifica API - ANNCSU – Consultazione per le PA.yaml"
        )

    @pytest.fixture
    def validator(self, spec_path):
        """Fixture providing ResponseValidator instance."""
        return ResponseValidator(spec_path)

    def test_elencoaccessiprog_get_query_param_success(self, validator):
        """Test validation of elencoaccessiprogGetQueryParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoaccessiprog",
            "data": [
                {
                    "prognazacc": "6744962",
                    "civico": "1",
                    "esp": "A",
                    "specif": "ROSSO",
                    "metrico": "1200",
                    "coordX": "12.5349017",
                    "coordY": "44.0191923",
                    "quota": "50",
                    "metodo": "4",
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "elencoaccessiprogGetQueryParam"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_elencoaccessiprog_post_success(self, validator):
        """Test validation of elencoaccessiprogPost 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoaccessiprog",
            "data": [
                {
                    "prognazacc": "1234567",
                    "civico": "42",
                    "esp": "",
                    "specif": "",
                    "metrico": "3000",
                    "coordX": "11.2567890",
                    "coordY": "43.7692105",
                    "quota": "100",
                    "metodo": "3",
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "elencoaccessiprogPost"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_elencoaccessiprog_get_path_param_success(self, validator):
        """Test validation of elencoaccessiprogGetPathParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoaccessiprog",
            "data": [
                {
                    "prognazacc": "9876543",
                    "civico": "100",
                    "esp": "C",
                    "specif": "INTERNO 5",
                    "metrico": "8000",
                    "coordX": "12.4963655",
                    "coordY": "41.9027835",
                    "quota": "25",
                    "metodo": "2",
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "elencoaccessiprogGetPathParam"
        )

        assert is_valid is True
        assert len(errors) == 0


class TestPrognazAreaOperations:
    """Tests for prognazarea operations (get street by national progressive)."""

    @pytest.fixture
    def spec_path(self):
        """Fixture providing path to OpenAPI spec."""
        return (
            Path(__file__).parent.parent.parent
            / "oas"
            / "dev"
            / "Specifica API - ANNCSU – Consultazione per le PA.yaml"
        )

    @pytest.fixture
    def validator(self, spec_path):
        """Fixture providing ResponseValidator instance."""
        return ResponseValidator(spec_path)

    def test_prognazarea_get_query_param_success(self, validator):
        """Test validation of prognazareaGetQueryParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "prognazarea",
            "data": [
                {
                    "prognaz": "919572",
                    "dug": "AUTOSTRADA",
                    "denomuff": "ROMA AEROPORTO DI FIUMICINO",
                    "denomloc": None,
                    "denomlingua1": None,
                    "denomlingua2": None,
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "prognazareaGetQueryParam"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_prognazarea_post_success(self, validator):
        """Test validation of prognazareaPost 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "prognazarea",
            "data": [
                {
                    "prognaz": "111222",
                    "dug": "VIA",
                    "denomuff": "NAZIONALE",
                    "denomloc": None,
                    "denomlingua1": None,
                    "denomlingua2": None,
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(response, "prognazareaPost")

        assert is_valid is True
        assert len(errors) == 0

    def test_prognazarea_get_path_param_success(self, validator):
        """Test validation of prognazareaGetPathParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "prognazarea",
            "data": [
                {
                    "prognaz": "333444",
                    "dug": "PIAZZA",
                    "denomuff": "DEL POPOLO",
                    "denomloc": "P.ZA DEL POPOLO",
                    "denomlingua1": None,
                    "denomlingua2": None,
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "prognazareaGetPathParam"
        )

        assert is_valid is True
        assert len(errors) == 0


class TestPrognazAccOperations:
    """Tests for prognazacc operations (get access by national progressive access ID)."""

    @pytest.fixture
    def spec_path(self):
        """Fixture providing path to OpenAPI spec."""
        return (
            Path(__file__).parent.parent.parent
            / "oas"
            / "dev"
            / "Specifica API - ANNCSU – Consultazione per le PA.yaml"
        )

    @pytest.fixture
    def validator(self, spec_path):
        """Fixture providing ResponseValidator instance."""
        return ResponseValidator(spec_path)

    def test_prognazacc_get_query_param_success(self, validator):
        """Test validation of prognazaccGetQueryParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoodonimiprog",
            "data": [
                {
                    "prognaz": "919572",
                    "dug": "AUTOSTRADA",
                    "denomuff": "ROMA AEROPORTO DI FIUMICINO",
                    "denomloc": None,
                    "denomlingua1": None,
                    "denomlingua2": None,
                    "prognazacc": "6744962",
                    "civico": "1",
                    "esp": "A",
                    "specif": "ROSSO",
                    "metrico": "1200",
                    "coordX": "12.5349017",
                    "coordY": "44.0191923",
                    "quota": "50",
                    "metodo": "4",
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "prognazaccGetQueryParam"
        )

        assert is_valid is True
        assert len(errors) == 0

    def test_prognazacc_post_success(self, validator):
        """Test validation of prognazaccPost 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoodonimiprog",
            "data": [
                {
                    "prognaz": "123456",
                    "dug": "VIA",
                    "denomuff": "CAVOUR",
                    "denomloc": None,
                    "denomlingua1": None,
                    "denomlingua2": None,
                    "prognazacc": "7891011",
                    "civico": "25",
                    "esp": "",
                    "specif": "",
                    "metrico": "2400",
                    "coordX": "11.3426163",
                    "coordY": "44.4938100",
                    "quota": "75",
                    "metodo": "3",
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(response, "prognazaccPost")

        assert is_valid is True
        assert len(errors) == 0

    def test_prognazacc_get_path_param_success(self, validator):
        """Test validation of prognazaccGetPathParam 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/json"}
        response_data = {
            "res": "elencoodonimiprog",
            "data": [
                {
                    "prognaz": "789012",
                    "dug": "CORSO",
                    "denomuff": "ITALIA",
                    "denomloc": "C. ITALIA",
                    "denomlingua1": None,
                    "denomlingua2": None,
                    "prognazacc": "1213141",
                    "civico": "50",
                    "esp": "B",
                    "specif": "BIS",
                    "metrico": "4200",
                    "coordX": "9.1859243",
                    "coordY": "45.4654219",
                    "quota": "120",
                    "metodo": "1",
                }
            ],
        }
        response.content = str(response_data).encode()
        response.json.return_value = response_data

        is_valid, errors = validator.validate_response(
            response, "prognazaccGetPathParam"
        )

        assert is_valid is True
        assert len(errors) == 0


class TestStatusOperation:
    """Tests for status operation (application health check)."""

    @pytest.fixture
    def spec_path(self):
        """Fixture providing path to OpenAPI spec."""
        return (
            Path(__file__).parent.parent.parent
            / "oas"
            / "dev"
            / "Specifica API - ANNCSU – Consultazione per le PA.yaml"
        )

    @pytest.fixture
    def validator(self, spec_path):
        """Fixture providing ResponseValidator instance."""
        return ResponseValidator(spec_path)

    def test_status_success_200(self, validator):
        """Test validation of show_status 200 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 200
        response.headers = {"content-type": "application/problem+json"}
        response.content = b'{"status": "OK"}'
        response.json.return_value = {"status": "OK"}

        is_valid, errors = validator.validate_response(response, "show_status")

        assert is_valid is True
        assert len(errors) == 0

    def test_status_service_unavailable_503(self, validator):
        """Test validation of show_status 503 response."""
        response = Mock(spec=httpx.Response)
        response.status_code = 503
        response.headers = {"content-type": "application/problem+json"}
        response.content = b'{"status": "Service Unavailable"}'
        response.json.return_value = {"status": "Service Unavailable"}

        is_valid, errors = validator.validate_response(response, "show_status")

        assert is_valid is True
        assert len(errors) == 0


class TestErrorResponsesAllOperations:
    """Tests for error responses across all operations."""

    @pytest.fixture
    def spec_path(self):
        """Fixture providing path to OpenAPI spec."""
        return (
            Path(__file__).parent.parent.parent
            / "oas"
            / "dev"
            / "Specifica API - ANNCSU – Consultazione per le PA.yaml"
        )

    @pytest.fixture
    def validator(self, spec_path):
        """Fixture providing ResponseValidator instance."""
        return ResponseValidator(spec_path)

    @pytest.mark.parametrize(
        "operation_id",
        [
            "esisteOdonimoGetQueryParam",
            "esisteAccessoPost",
            "elencoOdonimiGetPathParam",
            "elencoAccessiGetQueryParam",
            "elencoodonimiprogPost",
            "elencoaccessiprogGetPathParam",
            "prognazareaGetQueryParam",
            "prognazaccPost",
        ],
    )
    def test_400_bad_request_error(self, validator, operation_id):
        """Test validation of 400 Bad Request error for various operations."""
        response = Mock(spec=httpx.Response)
        response.status_code = 400
        response.headers = {"content-type": "application/problem+json"}
        response.content = (
            b'{"title": "Bad Request", "detail": "Invalid parameter value"}'
        )
        response.json.return_value = {
            "title": "Bad Request",
            "detail": "Invalid parameter value",
        }

        is_valid, errors = validator.validate_response(response, operation_id)

        assert is_valid is True
        assert len(errors) == 0

    @pytest.mark.parametrize(
        "operation_id",
        [
            "elencoOdonimiGetQueryParam",
            "elencoAccessiPost",
            "elencoodonimiprogGetPathParam",
            "elencoaccessiprogGetQueryParam",
            "prognazareaPost",
            "prognazaccGetPathParam",
        ],
    )
    def test_404_not_found_error(self, validator, operation_id):
        """Test validation of 404 Not Found error for list operations."""
        response = Mock(spec=httpx.Response)
        response.status_code = 404
        response.headers = {"content-type": "application/problem+json"}
        response.content = b'{"title": "Not Found", "detail": "Resource not found"}'
        response.json.return_value = {
            "title": "Not Found",
            "detail": "Resource not found",
        }

        is_valid, errors = validator.validate_response(response, operation_id)

        assert is_valid is True
        assert len(errors) == 0

    @pytest.mark.parametrize(
        "operation_id",
        [
            "esisteOdonimoPost",
            "esisteAccessoPost",
            "elencoOdonimiPost",
            "elencoAccessiPost",
            "elencoodonimiprogPost",
            "elencoaccessiprogPost",
            "prognazareaPost",
            "prognazaccPost",
        ],
    )
    def test_422_unprocessable_entity_error(self, validator, operation_id):
        """Test validation of 422 Unprocessable Entity error for POST operations."""
        response = Mock(spec=httpx.Response)
        response.status_code = 422
        response.headers = {"content-type": "application/problem+json"}
        response.content = (
            b'{"title": "Unprocessable Entity", "detail": "Invalid JSON body"}'
        )
        response.json.return_value = {
            "title": "Unprocessable Entity",
            "detail": "Invalid JSON body",
        }

        is_valid, errors = validator.validate_response(response, operation_id)

        assert is_valid is True
        assert len(errors) == 0

    @pytest.mark.parametrize(
        "operation_id",
        [
            "esisteOdonimoGetQueryParam",
            "elencoOdonimiPost",
            "elencoaccessiprogGetPathParam",
            "prognazareaGetQueryParam",
        ],
    )
    def test_500_internal_server_error(self, validator, operation_id):
        """Test validation of 500 Internal Server Error."""
        response = Mock(spec=httpx.Response)
        response.status_code = 500
        response.headers = {"content-type": "application/problem+json"}
        response.content = (
            b'{"title": "Internal Server Error", "detail": "Server error occurred"}'
        )
        response.json.return_value = {
            "title": "Internal Server Error",
            "detail": "Server error occurred",
        }

        is_valid, errors = validator.validate_response(response, operation_id)

        assert is_valid is True
        assert len(errors) == 0



"""Custom validators for ANNCSU-specific business rules."""

import base64
import re
from typing import Any


def base64_validator(v: Any) -> str:
    """
    Validate that a string is valid base64 encoded.

    Args:
        v: Value to validate

    Returns:
        The validated string

    Raises:
        ValueError: If the string is not valid base64
    """
    if not isinstance(v, str):
        raise ValueError("Value must be a string")

    try:
        # Validate base64 encoding
        base64.b64decode(v, validate=True)
        return v
    except Exception as e:
        raise ValueError(f"Invalid base64 encoding: {e}") from e


def belfiore_code_validator(v: Any) -> str:
    """
    Validate Italian Belfiore municipality code format.

    Format: One uppercase letter followed by 3 digits (e.g., H501 for Rome)

    Args:
        v: Value to validate

    Returns:
        The validated string

    Raises:
        ValueError: If the code doesn't match Belfiore format
    """
    if not isinstance(v, str):
        raise ValueError("Belfiore code must be a string")

    pattern = r"^[A-Z]\d{3}$"
    if not re.match(pattern, v):
        raise ValueError(
            f"Invalid Belfiore code format: '{v}'. "
            "Expected format: one uppercase letter followed by 3 digits (e.g., H501)"
        )

    return v



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from anncsu.common.types import BaseModel
from anncsu.common.utils import FieldMetadata, PathParamMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EsisteOdonimoGetPathParamRequestTypedDict(TypedDict):
    codcom: str
    r"""Codice Belfiore del comune dell'odonimo"""
    denom: str
    r"""Denominazione esatta dell'odonimo - base64 encoded"""


class EsisteOdonimoGetPathParamRequest(BaseModel):
    codcom: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Codice Belfiore del comune dell'odonimo"""

    denom: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Denominazione esatta dell'odonimo - base64 encoded"""


class EsisteOdonimoGetPathParamResponseTypedDict(TypedDict):
    r"""Successful operation"""

    res: NotRequired[str]
    data: NotRequired[bool]


class EsisteOdonimoGetPathParamResponse(BaseModel):
    r"""Successful operation"""

    res: Optional[str] = None

    data: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from anncsu.common.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ShowStatusResponseTypedDict(TypedDict):
    r"""Il server ha ritornato lo status."""

    status: NotRequired[str]


class ShowStatusResponse(BaseModel):
    r"""Il server ha ritornato lo status."""

    status: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from anncsu.common.errors import AnncsuBaseError as AnncsuError
from anncsu.common.types import BaseModel
import httpx
from typing import Optional


class ElencoAccessiPostInternalServerErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class ElencoAccessiPostInternalServerError(AnncsuError):
    r"""Internal Server Error"""

    data: ElencoAccessiPostInternalServerErrorData

    def __init__(
        self,
        data: ElencoAccessiPostInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class ElencoAccessiPostUnprocessableEntityErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class ElencoAccessiPostUnprocessableEntityError(AnncsuError):
    r"""Unprocessable Entity - error in json"""

    data: ElencoAccessiPostUnprocessableEntityErrorData

    def __init__(
        self,
        data: ElencoAccessiPostUnprocessableEntityErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class ElencoAccessiPostMethodNotAllowedErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class ElencoAccessiPostMethodNotAllowedError(AnncsuError):
    r"""Method Not Allowed"""

    data: ElencoAccessiPostMethodNotAllowedErrorData

    def __init__(
        self,
        data: ElencoAccessiPostMethodNotAllowedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class ElencoAccessiPostNotFoundErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class ElencoAccessiPostNotFoundError(AnncsuError):
    r"""Not found"""

    data: ElencoAccessiPostNotFoundErrorData

    def __init__(
        self,
        data: ElencoAccessiPostNotFoundErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class ElencoAccessiPostBadRequestErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class ElencoAccessiPostBadRequestError(AnncsuError):
    r"""Bad Request"""

    data: ElencoAccessiPostBadRequestErrorData

    def __init__(
        self,
        data: ElencoAccessiPostBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from anncsu.common.errors import AnncsuBaseError as AnncsuError
from anncsu.common.types import BaseModel
import httpx
from typing import Optional


class ElencoodonimiprogGetQueryParamInternalServerErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class ElencoodonimiprogGetQueryParamInternalServerError(AnncsuError):
    r"""Internal Server Error"""

    data: ElencoodonimiprogGetQueryParamInternalServerErrorData

    def __init__(
        self,
        data: ElencoodonimiprogGetQueryParamInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class ElencoodonimiprogGetQueryParamUnprocessableEntityErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class ElencoodonimiprogGetQueryParamUnprocessableEntityError(AnncsuError):
    r"""Unprocessable Entity - error in json"""

    data: ElencoodonimiprogGetQueryParamUnprocessableEntityErrorData

    def __init__(
        self,
        data: ElencoodonimiprogGetQueryParamUnprocessableEntityErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class ElencoodonimiprogGetQueryParamMethodNotAllowedErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class ElencoodonimiprogGetQueryParamMethodNotAllowedError(AnncsuError):
    r"""Method Not Allowed"""

    data: ElencoodonimiprogGetQueryParamMethodNotAllowedErrorData

    def __init__(
        self,
        data: ElencoodonimiprogGetQueryParamMethodNotAllowedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class ElencoodonimiprogGetQueryParamNotFoundErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class ElencoodonimiprogGetQueryParamNotFoundError(AnncsuError):
    r"""Not found"""

    data: ElencoodonimiprogGetQueryParamNotFoundErrorData

    def __init__(
        self,
        data: ElencoodonimiprogGetQueryParamNotFoundErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class ElencoodonimiprogGetQueryParamBadRequestErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class ElencoodonimiprogGetQueryParamBadRequestError(AnncsuError):
    r"""Bad Request"""

    data: ElencoodonimiprogGetQueryParamBadRequestErrorData

    def __init__(
        self,
        data: ElencoodonimiprogGetQueryParamBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import TYPE_CHECKING
from importlib import import_module

if TYPE_CHECKING:
    from .annotations import get_discriminator
    from .datetimes import parse_datetime
    from .enums import OpenEnumMeta
    from .headers import get_headers, get_response_headers
    from .metadata import (
        FieldMetadata,
        find_metadata,
        FormMetadata,
        HeaderMetadata,
        MultipartFormMetadata,
        PathParamMetadata,
        QueryParamMetadata,
        RequestMetadata,
        SecurityMetadata,
    )
    from .queryparams import get_query_params
    from .retries import BackoffStrategy, Retries, retry, retry_async, RetryConfig
    from .requestbodies import serialize_request_body, SerializedRequestBody
    from .security import get_security
    from .serializers import (
        get_pydantic_model,
        marshal_json,
        unmarshal,
        unmarshal_json,
        serialize_decimal,
        serialize_float,
        serialize_int,
        stream_to_text,
        stream_to_text_async,
        stream_to_bytes,
        stream_to_bytes_async,
        validate_const,
        validate_decimal,
        validate_float,
        validate_int,
        validate_open_enum,
    )
    from .url import generate_url, template_url, remove_suffix
    from .values import (
        get_global_from_env,
        match_content_type,
        match_status_codes,
        match_response,
        cast_partial,
    )
    from .logger import Logger, get_body_content, get_default_logger

__all__ = [
    "BackoffStrategy",
    "FieldMetadata",
    "find_metadata",
    "FormMetadata",
    "generate_url",
    "get_body_content",
    "get_default_logger",
    "get_discriminator",
    "parse_datetime",
    "get_global_from_env",
    "get_headers",
    "get_pydantic_model",
    "get_query_params",
    "get_response_headers",
    "get_security",
    "HeaderMetadata",
    "Logger",
    "marshal_json",
    "match_content_type",
    "match_status_codes",
    "match_response",
    "MultipartFormMetadata",
    "OpenEnumMeta",
    "PathParamMetadata",
    "QueryParamMetadata",
    "remove_suffix",
    "Retries",
    "retry",
    "retry_async",
    "RetryConfig",
    "RequestMetadata",
    "SecurityMetadata",
    "serialize_decimal",
    "serialize_float",
    "serialize_int",
    "serialize_request_body",
    "SerializedRequestBody",
    "stream_to_text",
    "stream_to_text_async",
    "stream_to_bytes",
    "stream_to_bytes_async",
    "template_url",
    "unmarshal",
    "unmarshal_json",
    "validate_decimal",
    "validate_const",
    "validate_float",
    "validate_int",
    "validate_open_enum",
    "cast_partial",
]

_dynamic_imports: dict[str, str] = {
    "BackoffStrategy": ".retries",
    "FieldMetadata": ".metadata",
    "find_metadata": ".metadata",
    "FormMetadata": ".metadata",
    "generate_url": ".url",
    "get_body_content": ".logger",
    "get_default_logger": ".logger",
    "get_discriminator": ".annotations",
    "parse_datetime": ".datetimes",
    "get_global_from_env": ".values",
    "get_headers": ".headers",
    "get_pydantic_model": ".serializers",
    "get_query_params": ".queryparams",
    "get_response_headers": ".headers",
    "get_security": ".security",
    "HeaderMetadata": ".metadata",
    "Logger": ".logger",
    "marshal_json": ".serializers",
    "match_content_type": ".values",
    "match_status_codes": ".values",
    "match_response": ".values",
    "MultipartFormMetadata": ".metadata",
    "OpenEnumMeta": ".enums",
    "PathParamMetadata": ".metadata",
    "QueryParamMetadata": ".metadata",
    "remove_suffix": ".url",
    "Retries": ".retries",
    "retry": ".retries",
    "retry_async": ".retries",
    "RetryConfig": ".retries",
    "RequestMetadata": ".metadata",
    "SecurityMetadata": ".metadata",
    "serialize_decimal": ".serializers",
    "serialize_float": ".serializers",
    "serialize_int": ".serializers",
    "serialize_request_body": ".requestbodies",
    "SerializedRequestBody": ".requestbodies",
    "stream_to_text": ".serializers",
    "stream_to_text_async": ".serializers",
    "stream_to_bytes": ".serializers",
    "stream_to_bytes_async": ".serializers",
    "template_url": ".url",
    "unmarshal": ".serializers",
    "unmarshal_json": ".serializers",
    "validate_decimal": ".serializers",
    "validate_const": ".serializers",
    "validate_float": ".serializers",
    "validate_int": ".serializers",
    "validate_open_enum": ".serializers",
    "cast_partial": ".values",
}


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"no {attr_name} found in _dynamic_imports, module name -> {__name__} "
        )

    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from anncsu.common.types import BaseModel
from anncsu.common.utils import FieldMetadata, QueryParamMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EsisteAccessoGetQueryParamRequestTypedDict(TypedDict):
    codcom: str
    r"""Codice Belfiore del comune dell'odonimo"""
    denom: str
    r"""Denominazione esatta dell'odonimo - base64 encoded"""
    accesso: str
    r"""valore civico(+eventuale esponente e/o specificit�) oppure metrico"""


class EsisteAccessoGetQueryParamRequest(BaseModel):
    codcom: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Codice Belfiore del comune dell'odonimo"""

    denom: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Denominazione esatta dell'odonimo - base64 encoded"""

    accesso: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""valore civico(+eventuale esponente e/o specificit�) oppure metrico"""


class EsisteAccessoGetQueryParamResponseTypedDict(TypedDict):
    r"""Successful operation"""

    res: NotRequired[str]
    data: NotRequired[bool]


class EsisteAccessoGetQueryParamResponse(BaseModel):
    r"""Successful operation"""

    res: Optional[str] = None

    data: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import TYPE_CHECKING
from importlib import import_module

if TYPE_CHECKING:
    from .elencoaccessigetpathparamop import (
        ElencoAccessiGetPathParamData,
        ElencoAccessiGetPathParamDataTypedDict,
        ElencoAccessiGetPathParamRequest,
        ElencoAccessiGetPathParamRequestTypedDict,
        ElencoAccessiGetPathParamResponse,
        ElencoAccessiGetPathParamResponseTypedDict,
    )
    from .elencoaccessigetqueryparamop import (
        ElencoAccessiGetQueryParamData,
        ElencoAccessiGetQueryParamDataTypedDict,
        ElencoAccessiGetQueryParamRequest,
        ElencoAccessiGetQueryParamRequestTypedDict,
        ElencoAccessiGetQueryParamResponse,
        ElencoAccessiGetQueryParamResponseTypedDict,
    )
    from .elencoaccessipostop import (
        ElencoAccessiPostData,
        ElencoAccessiPostDataTypedDict,
        ElencoAccessiPostRequest,
        ElencoAccessiPostRequestTypedDict,
        ElencoAccessiPostResponse,
        ElencoAccessiPostResponseTypedDict,
    )
    from .elencoaccessiproggetpathparamop import (
        ElencoaccessiprogGetPathParamData,
        ElencoaccessiprogGetPathParamDataTypedDict,
        ElencoaccessiprogGetPathParamRequest,
        ElencoaccessiprogGetPathParamRequestTypedDict,
        ElencoaccessiprogGetPathParamResponse,
        ElencoaccessiprogGetPathParamResponseTypedDict,
    )
    from .elencoaccessiproggetqueryparamop import (
        ElencoaccessiprogGetQueryParamData,
        ElencoaccessiprogGetQueryParamDataTypedDict,
        ElencoaccessiprogGetQueryParamRequest,
        ElencoaccessiprogGetQueryParamRequestTypedDict,
        ElencoaccessiprogGetQueryParamResponse,
        ElencoaccessiprogGetQueryParamResponseTypedDict,
    )
    from .elencoaccessiprogpostop import (
        ElencoaccessiprogPostData,
        ElencoaccessiprogPostDataTypedDict,
        ElencoaccessiprogPostRequest,
        ElencoaccessiprogPostRequestTypedDict,
        ElencoaccessiprogPostResponse,
        ElencoaccessiprogPostResponseTypedDict,
    )
    from .elencoodonimigetpathparamop import (
        ElencoOdonimiGetPathParamData,
        ElencoOdonimiGetPathParamDataTypedDict,
        ElencoOdonimiGetPathParamRequest,
        ElencoOdonimiGetPathParamRequestTypedDict,
        ElencoOdonimiGetPathParamResponse,
        ElencoOdonimiGetPathParamResponseTypedDict,
    )
    from .elencoodonimigetqueryparamop import (
        ElencoOdonimiGetQueryParamData,
        ElencoOdonimiGetQueryParamDataTypedDict,
        ElencoOdonimiGetQueryParamRequest,
        ElencoOdonimiGetQueryParamRequestTypedDict,
        ElencoOdonimiGetQueryParamResponse,
        ElencoOdonimiGetQueryParamResponseTypedDict,
    )
    from .elencoodonimipostop import (
        ElencoOdonimiPostData,
        ElencoOdonimiPostDataTypedDict,
        ElencoOdonimiPostRequest,
        ElencoOdonimiPostRequestTypedDict,
        ElencoOdonimiPostResponse,
        ElencoOdonimiPostResponseTypedDict,
    )
    from .elencoodonimiproggetpathparamop import (
        ElencoodonimiprogGetPathParamData,
        ElencoodonimiprogGetPathParamDataTypedDict,
        ElencoodonimiprogGetPathParamRequest,
        ElencoodonimiprogGetPathParamRequestTypedDict,
        ElencoodonimiprogGetPathParamResponse,
        ElencoodonimiprogGetPathParamResponseTypedDict,
    )
    from .elencoodonimiproggetqueryparamop import (
        ElencoodonimiprogGetQueryParamData,
        ElencoodonimiprogGetQueryParamDataTypedDict,
        ElencoodonimiprogGetQueryParamRequest,
        ElencoodonimiprogGetQueryParamRequestTypedDict,
        ElencoodonimiprogGetQueryParamResponse,
        ElencoodonimiprogGetQueryParamResponseTypedDict,
    )
    from .elencoodonimiprogpostop import (
        ElencoodonimiprogPostData,
        ElencoodonimiprogPostDataTypedDict,
        ElencoodonimiprogPostRequest,
        ElencoodonimiprogPostRequestTypedDict,
        ElencoodonimiprogPostResponse,
        ElencoodonimiprogPostResponseTypedDict,
    )
    from .esisteaccessogetpathparamop import (
        EsisteAccessoGetPathParamRequest,
        EsisteAccessoGetPathParamRequestTypedDict,
        EsisteAccessoGetPathParamResponse,
        EsisteAccessoGetPathParamResponseTypedDict,
    )
    from .esisteaccessogetqueryparamop import (
        EsisteAccessoGetQueryParamRequest,
        EsisteAccessoGetQueryParamRequestTypedDict,
        EsisteAccessoGetQueryParamResponse,
        EsisteAccessoGetQueryParamResponseTypedDict,
    )
    from .esisteaccessopostop import (
        EsisteAccessoPostRequest,
        EsisteAccessoPostRequestTypedDict,
        EsisteAccessoPostResponse,
        EsisteAccessoPostResponseTypedDict,
    )
    from .esisteodonimogetpathparamop import (
        EsisteOdonimoGetPathParamRequest,
        EsisteOdonimoGetPathParamRequestTypedDict,
        EsisteOdonimoGetPathParamResponse,
        EsisteOdonimoGetPathParamResponseTypedDict,
    )
    from .esisteodonimogetqueryparamop import (
        EsisteOdonimoGetQueryParamRequest,
        EsisteOdonimoGetQueryParamRequestTypedDict,
        EsisteOdonimoGetQueryParamResponse,
        EsisteOdonimoGetQueryParamResponseTypedDict,
    )
    from .esisteodonimopostop import (
        EsisteOdonimoPostRequest,
        EsisteOdonimoPostRequestTypedDict,
        EsisteOdonimoPostResponse,
        EsisteOdonimoPostResponseTypedDict,
    )
    from .prognazaccgetpathparamop import (
        PrognazaccGetPathParamData,
        PrognazaccGetPathParamDataTypedDict,
        PrognazaccGetPathParamRequest,
        PrognazaccGetPathParamRequestTypedDict,
        PrognazaccGetPathParamResponse,
        PrognazaccGetPathParamResponseTypedDict,
    )
    from .prognazaccgetqueryparamop import (
        PrognazaccGetQueryParamData,
        PrognazaccGetQueryParamDataTypedDict,
        PrognazaccGetQueryParamRequest,
        PrognazaccGetQueryParamRequestTypedDict,
        PrognazaccGetQueryParamResponse,
        PrognazaccGetQueryParamResponseTypedDict,
    )
    from .prognazaccpostop import (
        PrognazaccPostData,
        PrognazaccPostDataTypedDict,
        PrognazaccPostRequest,
        PrognazaccPostRequestTypedDict,
        PrognazaccPostResponse,
        PrognazaccPostResponseTypedDict,
    )
    from .prognazareagetpathparamop import (
        PrognazareaGetPathParamData,
        PrognazareaGetPathParamDataTypedDict,
        PrognazareaGetPathParamRequest,
        PrognazareaGetPathParamRequestTypedDict,
        PrognazareaGetPathParamResponse,
        PrognazareaGetPathParamResponseTypedDict,
    )
    from .prognazareagetqueryparamop import (
        PrognazareaGetQueryParamData,
        PrognazareaGetQueryParamDataTypedDict,
        PrognazareaGetQueryParamRequest,
        PrognazareaGetQueryParamRequestTypedDict,
        PrognazareaGetQueryParamResponse,
        PrognazareaGetQueryParamResponseTypedDict,
    )
    from .prognazareapostop import (
        PrognazareaPostData,
        PrognazareaPostDataTypedDict,
        PrognazareaPostRequest,
        PrognazareaPostRequestTypedDict,
        PrognazareaPostResponse,
        PrognazareaPostResponseTypedDict,
    )
    from .show_statusop import ShowStatusResponse, ShowStatusResponseTypedDict

__all__ = [
    "ElencoAccessiGetPathParamData",
    "ElencoAccessiGetPathParamDataTypedDict",
    "ElencoAccessiGetPathParamRequest",
    "ElencoAccessiGetPathParamRequestTypedDict",
    "ElencoAccessiGetPathParamResponse",
    "ElencoAccessiGetPathParamResponseTypedDict",
    "ElencoAccessiGetQueryParamData",
    "ElencoAccessiGetQueryParamDataTypedDict",
    "ElencoAccessiGetQueryParamRequest",
    "ElencoAccessiGetQueryParamRequestTypedDict",
    "ElencoAccessiGetQueryParamResponse",
    "ElencoAccessiGetQueryParamResponseTypedDict",
    "ElencoAccessiPostData",
    "ElencoAccessiPostDataTypedDict",
    "ElencoAccessiPostRequest",
    "ElencoAccessiPostRequestTypedDict",
    "ElencoAccessiPostResponse",
    "ElencoAccessiPostResponseTypedDict",
    "ElencoOdonimiGetPathParamData",
    "ElencoOdonimiGetPathParamDataTypedDict",
    "ElencoOdonimiGetPathParamRequest",
    "ElencoOdonimiGetPathParamRequestTypedDict",
    "ElencoOdonimiGetPathParamResponse",
    "ElencoOdonimiGetPathParamResponseTypedDict",
    "ElencoOdonimiGetQueryParamData",
    "ElencoOdonimiGetQueryParamDataTypedDict",
    "ElencoOdonimiGetQueryParamRequest",
    "ElencoOdonimiGetQueryParamRequestTypedDict",
    "ElencoOdonimiGetQueryParamResponse",
    "ElencoOdonimiGetQueryParamResponseTypedDict",
    "ElencoOdonimiPostData",
    "ElencoOdonimiPostDataTypedDict",
    "ElencoOdonimiPostRequest",
    "ElencoOdonimiPostRequestTypedDict",
    "ElencoOdonimiPostResponse",
    "ElencoOdonimiPostResponseTypedDict",
    "ElencoaccessiprogGetPathParamData",
    "ElencoaccessiprogGetPathParamDataTypedDict",
    "ElencoaccessiprogGetPathParamRequest",
    "ElencoaccessiprogGetPathParamRequestTypedDict",
    "ElencoaccessiprogGetPathParamResponse",
    "ElencoaccessiprogGetPathParamResponseTypedDict",
    "ElencoaccessiprogGetQueryParamData",
    "ElencoaccessiprogGetQueryParamDataTypedDict",
    "ElencoaccessiprogGetQueryParamRequest",
    "ElencoaccessiprogGetQueryParamRequestTypedDict",
    "ElencoaccessiprogGetQueryParamResponse",
    "ElencoaccessiprogGetQueryParamResponseTypedDict",
    "ElencoaccessiprogPostData",
    "ElencoaccessiprogPostDataTypedDict",
    "ElencoaccessiprogPostRequest",
    "ElencoaccessiprogPostRequestTypedDict",
    "ElencoaccessiprogPostResponse",
    "ElencoaccessiprogPostResponseTypedDict",
    "ElencoodonimiprogGetPathParamData",
    "ElencoodonimiprogGetPathParamDataTypedDict",
    "ElencoodonimiprogGetPathParamRequest",
    "ElencoodonimiprogGetPathParamRequestTypedDict",
    "ElencoodonimiprogGetPathParamResponse",
    "ElencoodonimiprogGetPathParamResponseTypedDict",
    "ElencoodonimiprogGetQueryParamData",
    "ElencoodonimiprogGetQueryParamDataTypedDict",
    "ElencoodonimiprogGetQueryParamRequest",
    "ElencoodonimiprogGetQueryParamRequestTypedDict",
    "ElencoodonimiprogGetQueryParamResponse",
    "ElencoodonimiprogGetQueryParamResponseTypedDict",
    "ElencoodonimiprogPostData",
    "ElencoodonimiprogPostDataTypedDict",
    "ElencoodonimiprogPostRequest",
    "ElencoodonimiprogPostRequestTypedDict",
    "ElencoodonimiprogPostResponse",
    "ElencoodonimiprogPostResponseTypedDict",
    "EsisteAccessoGetPathParamRequest",
    "EsisteAccessoGetPathParamRequestTypedDict",
    "EsisteAccessoGetPathParamResponse",
    "EsisteAccessoGetPathParamResponseTypedDict",
    "EsisteAccessoGetQueryParamRequest",
    "EsisteAccessoGetQueryParamRequestTypedDict",
    "EsisteAccessoGetQueryParamResponse",
    "EsisteAccessoGetQueryParamResponseTypedDict",
    "EsisteAccessoPostRequest",
    "EsisteAccessoPostRequestTypedDict",
    "EsisteAccessoPostResponse",
    "EsisteAccessoPostResponseTypedDict",
    "EsisteOdonimoGetPathParamRequest",
    "EsisteOdonimoGetPathParamRequestTypedDict",
    "EsisteOdonimoGetPathParamResponse",
    "EsisteOdonimoGetPathParamResponseTypedDict",
    "EsisteOdonimoGetQueryParamRequest",
    "EsisteOdonimoGetQueryParamRequestTypedDict",
    "EsisteOdonimoGetQueryParamResponse",
    "EsisteOdonimoGetQueryParamResponseTypedDict",
    "EsisteOdonimoPostRequest",
    "EsisteOdonimoPostRequestTypedDict",
    "EsisteOdonimoPostResponse",
    "EsisteOdonimoPostResponseTypedDict",
    "PrognazaccGetPathParamData",
    "PrognazaccGetPathParamDataTypedDict",
    "PrognazaccGetPathParamRequest",
    "PrognazaccGetPathParamRequestTypedDict",
    "PrognazaccGetPathParamResponse",
    "PrognazaccGetPathParamResponseTypedDict",
    "PrognazaccGetQueryParamData",
    "PrognazaccGetQueryParamDataTypedDict",
    "PrognazaccGetQueryParamRequest",
    "PrognazaccGetQueryParamRequestTypedDict",
    "PrognazaccGetQueryParamResponse",
    "PrognazaccGetQueryParamResponseTypedDict",
    "PrognazaccPostData",
    "PrognazaccPostDataTypedDict",
    "PrognazaccPostRequest",
    "PrognazaccPostRequestTypedDict",
    "PrognazaccPostResponse",
    "PrognazaccPostResponseTypedDict",
    "PrognazareaGetPathParamData",
    "PrognazareaGetPathParamDataTypedDict",
    "PrognazareaGetPathParamRequest",
    "PrognazareaGetPathParamRequestTypedDict",
    "PrognazareaGetPathParamResponse",
    "PrognazareaGetPathParamResponseTypedDict",
    "PrognazareaGetQueryParamData",
    "PrognazareaGetQueryParamDataTypedDict",
    "PrognazareaGetQueryParamRequest",
    "PrognazareaGetQueryParamRequestTypedDict",
    "PrognazareaGetQueryParamResponse",
    "PrognazareaGetQueryParamResponseTypedDict",
    "PrognazareaPostData",
    "PrognazareaPostDataTypedDict",
    "PrognazareaPostRequest",
    "PrognazareaPostRequestTypedDict",
    "PrognazareaPostResponse",
    "PrognazareaPostResponseTypedDict",
    "ShowStatusResponse",
    "ShowStatusResponseTypedDict",
]

_dynamic_imports: dict[str, str] = {
    "ElencoAccessiGetPathParamData": ".elencoaccessigetpathparamop",
    "ElencoAccessiGetPathParamDataTypedDict": ".elencoaccessigetpathparamop",
    "ElencoAccessiGetPathParamRequest": ".elencoaccessigetpathparamop",
    "ElencoAccessiGetPathParamRequestTypedDict": ".elencoaccessigetpathparamop",
    "ElencoAccessiGetPathParamResponse": ".elencoaccessigetpathparamop",
    "ElencoAccessiGetPathParamResponseTypedDict": ".elencoaccessigetpathparamop",
    "ElencoAccessiGetQueryParamData": ".elencoaccessigetqueryparamop",
    "ElencoAccessiGetQueryParamDataTypedDict": ".elencoaccessigetqueryparamop",
    "ElencoAccessiGetQueryParamRequest": ".elencoaccessigetqueryparamop",
    "ElencoAccessiGetQueryParamRequestTypedDict": ".elencoaccessigetqueryparamop",
    "ElencoAccessiGetQueryParamResponse": ".elencoaccessigetqueryparamop",
    "ElencoAccessiGetQueryParamResponseTypedDict": ".elencoaccessigetqueryparamop",
    "ElencoAccessiPostData": ".elencoaccessipostop",
    "ElencoAccessiPostDataTypedDict": ".elencoaccessipostop",
    "ElencoAccessiPostRequest": ".elencoaccessipostop",
    "ElencoAccessiPostRequestTypedDict": ".elencoaccessipostop",
    "ElencoAccessiPostResponse": ".elencoaccessipostop",
    "ElencoAccessiPostResponseTypedDict": ".elencoaccessipostop",
    "ElencoaccessiprogGetPathParamData": ".elencoaccessiproggetpathparamop",
    "ElencoaccessiprogGetPathParamDataTypedDict": ".elencoaccessiproggetpathparamop",
    "ElencoaccessiprogGetPathParamRequest": ".elencoaccessiproggetpathparamop",
    "ElencoaccessiprogGetPathParamRequestTypedDict": ".elencoaccessiproggetpathparamop",
    "ElencoaccessiprogGetPathParamResponse": ".elencoaccessiproggetpathparamop",
    "ElencoaccessiprogGetPathParamResponseTypedDict": ".elencoaccessiproggetpathparamop",
    "ElencoaccessiprogGetQueryParamData": ".elencoaccessiproggetqueryparamop",
    "ElencoaccessiprogGetQueryParamDataTypedDict": ".elencoaccessiproggetqueryparamop",
    "ElencoaccessiprogGetQueryParamRequest": ".elencoaccessiproggetqueryparamop",
    "ElencoaccessiprogGetQueryParamRequestTypedDict": ".elencoaccessiproggetqueryparamop",
    "ElencoaccessiprogGetQueryParamResponse": ".elencoaccessiproggetqueryparamop",
    "ElencoaccessiprogGetQueryParamResponseTypedDict": ".elencoaccessiproggetqueryparamop",
    "ElencoaccessiprogPostData": ".elencoaccessiprogpostop",
    "ElencoaccessiprogPostDataTypedDict": ".elencoaccessiprogpostop",
    "ElencoaccessiprogPostRequest": ".elencoaccessiprogpostop",
    "ElencoaccessiprogPostRequestTypedDict": ".elencoaccessiprogpostop",
    "ElencoaccessiprogPostResponse": ".elencoaccessiprogpostop",
    "ElencoaccessiprogPostResponseTypedDict": ".elencoaccessiprogpostop",
    "ElencoOdonimiGetPathParamData": ".elencoodonimigetpathparamop",
    "ElencoOdonimiGetPathParamDataTypedDict": ".elencoodonimigetpathparamop",
    "ElencoOdonimiGetPathParamRequest": ".elencoodonimigetpathparamop",
    "ElencoOdonimiGetPathParamRequestTypedDict": ".elencoodonimigetpathparamop",
    "ElencoOdonimiGetPathParamResponse": ".elencoodonimigetpathparamop",
    "ElencoOdonimiGetPathParamResponseTypedDict": ".elencoodonimigetpathparamop",
    "ElencoOdonimiGetQueryParamData": ".elencoodonimigetqueryparamop",
    "ElencoOdonimiGetQueryParamDataTypedDict": ".elencoodonimigetqueryparamop",
    "ElencoOdonimiGetQueryParamRequest": ".elencoodonimigetqueryparamop",
    "ElencoOdonimiGetQueryParamRequestTypedDict": ".elencoodonimigetqueryparamop",
    "ElencoOdonimiGetQueryParamResponse": ".elencoodonimigetqueryparamop",
    "ElencoOdonimiGetQueryParamResponseTypedDict": ".elencoodonimigetqueryparamop",
    "ElencoOdonimiPostData": ".elencoodonimipostop",
    "ElencoOdonimiPostDataTypedDict": ".elencoodonimipostop",
    "ElencoOdonimiPostRequest": ".elencoodonimipostop",
    "ElencoOdonimiPostRequestTypedDict": ".elencoodonimipostop",
    "ElencoOdonimiPostResponse": ".elencoodonimipostop",
    "ElencoOdonimiPostResponseTypedDict": ".elencoodonimipostop",
    "ElencoodonimiprogGetPathParamData": ".elencoodonimiproggetpathparamop",
    "ElencoodonimiprogGetPathParamDataTypedDict": ".elencoodonimiproggetpathparamop",
    "ElencoodonimiprogGetPathParamRequest": ".elencoodonimiproggetpathparamop",
    "ElencoodonimiprogGetPathParamRequestTypedDict": ".elencoodonimiproggetpathparamop",
    "ElencoodonimiprogGetPathParamResponse": ".elencoodonimiproggetpathparamop",
    "ElencoodonimiprogGetPathParamResponseTypedDict": ".elencoodonimiproggetpathparamop",
    "ElencoodonimiprogGetQueryParamData": ".elencoodonimiproggetqueryparamop",
    "ElencoodonimiprogGetQueryParamDataTypedDict": ".elencoodonimiproggetqueryparamop",
    "ElencoodonimiprogGetQueryParamRequest": ".elencoodonimiproggetqueryparamop",
    "ElencoodonimiprogGetQueryParamRequestTypedDict": ".elencoodonimiproggetqueryparamop",
    "ElencoodonimiprogGetQueryParamResponse": ".elencoodonimiproggetqueryparamop",
    "ElencoodonimiprogGetQueryParamResponseTypedDict": ".elencoodonimiproggetqueryparamop",
    "ElencoodonimiprogPostData": ".elencoodonimiprogpostop",
    "ElencoodonimiprogPostDataTypedDict": ".elencoodonimiprogpostop",
    "ElencoodonimiprogPostRequest": ".elencoodonimiprogpostop",
    "ElencoodonimiprogPostRequestTypedDict": ".elencoodonimiprogpostop",
    "ElencoodonimiprogPostResponse": ".elencoodonimiprogpostop",
    "ElencoodonimiprogPostResponseTypedDict": ".elencoodonimiprogpostop",
    "EsisteAccessoGetPathParamRequest": ".esisteaccessogetpathparamop",
    "EsisteAccessoGetPathParamRequestTypedDict": ".esisteaccessogetpathparamop",
    "EsisteAccessoGetPathParamResponse": ".esisteaccessogetpathparamop",
    "EsisteAccessoGetPathParamResponseTypedDict": ".esisteaccessogetpathparamop",
    "EsisteAccessoGetQueryParamRequest": ".esisteaccessogetqueryparamop",
    "EsisteAccessoGetQueryParamRequestTypedDict": ".esisteaccessogetqueryparamop",
    "EsisteAccessoGetQueryParamResponse": ".esisteaccessogetqueryparamop",
    "EsisteAccessoGetQueryParamResponseTypedDict": ".esisteaccessogetqueryparamop",
    "EsisteAccessoPostRequest": ".esisteaccessopostop",
    "EsisteAccessoPostRequestTypedDict": ".esisteaccessopostop",
    "EsisteAccessoPostResponse": ".esisteaccessopostop",
    "EsisteAccessoPostResponseTypedDict": ".esisteaccessopostop",
    "EsisteOdonimoGetPathParamRequest": ".esisteodonimogetpathparamop",
    "EsisteOdonimoGetPathParamRequestTypedDict": ".esisteodonimogetpathparamop",
    "EsisteOdonimoGetPathParamResponse": ".esisteodonimogetpathparamop",
    "EsisteOdonimoGetPathParamResponseTypedDict": ".esisteodonimogetpathparamop",
    "EsisteOdonimoGetQueryParamRequest": ".esisteodonimogetqueryparamop",
    "EsisteOdonimoGetQueryParamRequestTypedDict": ".esisteodonimogetqueryparamop",
    "EsisteOdonimoGetQueryParamResponse": ".esisteodonimogetqueryparamop",
    "EsisteOdonimoGetQueryParamResponseTypedDict": ".esisteodonimogetqueryparamop",
    "EsisteOdonimoPostRequest": ".esisteodonimopostop",
    "EsisteOdonimoPostRequestTypedDict": ".esisteodonimopostop",
    "EsisteOdonimoPostResponse": ".esisteodonimopostop",
    "EsisteOdonimoPostResponseTypedDict": ".esisteodonimopostop",
    "PrognazaccGetPathParamData": ".prognazaccgetpathparamop",
    "PrognazaccGetPathParamDataTypedDict": ".prognazaccgetpathparamop",
    "PrognazaccGetPathParamRequest": ".prognazaccgetpathparamop",
    "PrognazaccGetPathParamRequestTypedDict": ".prognazaccgetpathparamop",
    "PrognazaccGetPathParamResponse": ".prognazaccgetpathparamop",
    "PrognazaccGetPathParamResponseTypedDict": ".prognazaccgetpathparamop",
    "PrognazaccGetQueryParamData": ".prognazaccgetqueryparamop",
    "PrognazaccGetQueryParamDataTypedDict": ".prognazaccgetqueryparamop",
    "PrognazaccGetQueryParamRequest": ".prognazaccgetqueryparamop",
    "PrognazaccGetQueryParamRequestTypedDict": ".prognazaccgetqueryparamop",
    "PrognazaccGetQueryParamResponse": ".prognazaccgetqueryparamop",
    "PrognazaccGetQueryParamResponseTypedDict": ".prognazaccgetqueryparamop",
    "PrognazaccPostData": ".prognazaccpostop",
    "PrognazaccPostDataTypedDict": ".prognazaccpostop",
    "PrognazaccPostRequest": ".prognazaccpostop",
    "PrognazaccPostRequestTypedDict": ".prognazaccpostop",
    "PrognazaccPostResponse": ".prognazaccpostop",
    "PrognazaccPostResponseTypedDict": ".prognazaccpostop",
    "PrognazareaGetPathParamData": ".prognazareagetpathparamop",
    "PrognazareaGetPathParamDataTypedDict": ".prognazareagetpathparamop",
    "PrognazareaGetPathParamRequest": ".prognazareagetpathparamop",
    "PrognazareaGetPathParamRequestTypedDict": ".prognazareagetpathparamop",
    "PrognazareaGetPathParamResponse": ".prognazareagetpathparamop",
    "PrognazareaGetPathParamResponseTypedDict": ".prognazareagetpathparamop",
    "PrognazareaGetQueryParamData": ".prognazareagetqueryparamop",
    "PrognazareaGetQueryParamDataTypedDict": ".prognazareagetqueryparamop",
    "PrognazareaGetQueryParamRequest": ".prognazareagetqueryparamop",
    "PrognazareaGetQueryParamRequestTypedDict": ".prognazareagetqueryparamop",
    "PrognazareaGetQueryParamResponse": ".prognazareagetqueryparamop",
    "PrognazareaGetQueryParamResponseTypedDict": ".prognazareagetqueryparamop",
    "PrognazareaPostData": ".prognazareapostop",
    "PrognazareaPostDataTypedDict": ".prognazareapostop",
    "PrognazareaPostRequest": ".prognazareapostop",
    "PrognazareaPostRequestTypedDict": ".prognazareapostop",
    "PrognazareaPostResponse": ".prognazareapostop",
    "PrognazareaPostResponseTypedDict": ".prognazareapostop",
    "ShowStatusResponse": ".show_statusop",
    "ShowStatusResponseTypedDict": ".show_statusop",
}


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)



"""Test imports for anncsu.common package.

This test suite verifies that all modules in the anncsu.common package
can be imported successfully after the refactoring that extracted shared
primitives from anncsu.pa.
"""

import pytest


class TestCommonPackageImports:
    """Test that the anncsu.common package structure is correct."""

    def test_import_common_package(self):
        """Test that anncsu.common package can be imported."""
        import anncsu.common

        assert anncsu.common is not None

    def test_common_package_has_path(self):
        """Test that anncsu.common has a __path__ attribute."""
        import anncsu.common

        assert hasattr(anncsu.common, "__path__")


class TestCommonTypesImports:
    """Test imports from anncsu.common.types module."""

    def test_import_types_module(self):
        """Test that anncsu.common.types can be imported."""
        import anncsu.common.types

        assert anncsu.common.types is not None

    def test_import_basemodel(self):
        """Test that BaseModel can be imported."""
        from anncsu.common.types import BaseModel

        assert BaseModel is not None

    def test_import_nullable(self):
        """Test that Nullable can be imported."""
        from anncsu.common.types import Nullable

        assert Nullable is not None

    def test_import_optional_nullable(self):
        """Test that OptionalNullable can be imported."""
        from anncsu.common.types import OptionalNullable

        assert OptionalNullable is not None

    def test_import_unset(self):
        """Test that UNSET can be imported."""
        from anncsu.common.types import UNSET

        assert UNSET is not None

    def test_import_unset_sentinel(self):
        """Test that UNSET_SENTINEL can be imported."""
        from anncsu.common.types import UNSET_SENTINEL

        assert UNSET_SENTINEL is not None

    def test_import_unrecognized_types(self):
        """Test that UnrecognizedInt and UnrecognizedStr can be imported."""
        from anncsu.common.types import UnrecognizedInt, UnrecognizedStr

        assert UnrecognizedInt is not None
        assert UnrecognizedStr is not None


class TestCommonUtilsImports:
    """Test imports from anncsu.common.utils module."""

    def test_import_utils_module(self):
        """Test that anncsu.common.utils can be imported."""
        import anncsu.common.utils

        assert anncsu.common.utils is not None

    def test_import_retry_config(self):
        """Test that RetryConfig can be imported."""
        from anncsu.common.utils import RetryConfig

        assert RetryConfig is not None

    def test_import_backoff_strategy(self):
        """Test that BackoffStrategy can be imported."""
        from anncsu.common.utils import BackoffStrategy

        assert BackoffStrategy is not None

    def test_import_serialized_request_body(self):
        """Test that SerializedRequestBody can be imported."""
        from anncsu.common.utils import SerializedRequestBody

        assert SerializedRequestBody is not None

    def test_import_field_metadata(self):
        """Test that FieldMetadata can be imported."""
        from anncsu.common.utils import FieldMetadata

        assert FieldMetadata is not None

    def test_import_query_param_metadata(self):
        """Test that QueryParamMetadata can be imported."""
        from anncsu.common.utils import QueryParamMetadata

        assert QueryParamMetadata is not None

    def test_import_path_param_metadata(self):
        """Test that PathParamMetadata can be imported."""
        from anncsu.common.utils import PathParamMetadata

        assert PathParamMetadata is not None

    def test_import_header_metadata(self):
        """Test that HeaderMetadata can be imported."""
        from anncsu.common.utils import HeaderMetadata

        assert HeaderMetadata is not None


class TestCommonErrorsImports:
    """Test imports from anncsu.common.errors module."""

    def test_import_errors_module(self):
        """Test that anncsu.common.errors can be imported."""
        import anncsu.common.errors

        assert anncsu.common.errors is not None

    def test_import_anncsu_base_error(self):
        """Test that AnncsuBaseError can be imported."""
        from anncsu.common.errors import AnncsuBaseError

        assert AnncsuBaseError is not None
        assert issubclass(AnncsuBaseError, Exception)

    def test_import_api_error(self):
        """Test that APIError can be imported."""
        from anncsu.common.errors import APIError

        assert APIError is not None

    def test_import_no_response_error(self):
        """Test that NoResponseError can be imported."""
        from anncsu.common.errors import NoResponseError

        assert NoResponseError is not None
        assert issubclass(NoResponseError, Exception)

    def test_import_response_validation_error(self):
        """Test that ResponseValidationError can be imported."""
        from anncsu.common.errors import ResponseValidationError

        assert ResponseValidationError is not None

    def test_api_error_inherits_from_base(self):
        """Test that APIError inherits from AnncsuBaseError."""
        from anncsu.common.errors import AnncsuBaseError, APIError

        assert issubclass(APIError, AnncsuBaseError)

    def test_response_validation_error_inherits_from_base(self):
        """Test that ResponseValidationError inherits from AnncsuBaseError."""
        from anncsu.common.errors import (
            AnncsuBaseError,
            ResponseValidationError,
        )

        assert issubclass(ResponseValidationError, AnncsuBaseError)


class TestCommonHooksImports:
    """Test imports from anncsu.common.hooks module."""

    def test_import_hooks_module(self):
        """Test that anncsu.common.hooks can be imported."""
        import anncsu.common.hooks

        assert anncsu.common.hooks is not None

    def test_import_sdk_hooks(self):
        """Test that SDKHooks can be imported."""
        from anncsu.common.hooks import SDKHooks

        assert SDKHooks is not None

    def test_import_hook_contexts(self):
        """Test that hook context classes can be imported."""
        from anncsu.common.hooks import (
            AfterErrorContext,
            AfterSuccessContext,
            BeforeRequestContext,
        )

        assert BeforeRequestContext is not None
        assert AfterSuccessContext is not None
        assert AfterErrorContext is not None


class TestCommonInfrastructureImports:
    """Test imports of infrastructure components."""

    def test_import_http_client(self):
        """Test that HTTPClient can be imported."""
        from anncsu.common.httpclient import HttpClient

        assert HttpClient is not None

    def test_import_async_http_client(self):
        """Test that AsyncHttpClient can be imported."""
        from anncsu.common.httpclient import AsyncHttpClient

        assert AsyncHttpClient is not None

    def test_import_base_sdk(self):
        """Test that BaseSDK can be imported."""
        from anncsu.common.basesdk import BaseSDK

        assert BaseSDK is not None

    def test_import_sdk_configuration(self):
        """Test that SDKConfiguration can be imported."""
        from anncsu.common.sdkconfiguration import SDKConfiguration

        assert SDKConfiguration is not None


class TestCommonUtilityModules:
    """Test that individual utility modules can be imported."""

    def test_import_annotations(self):
        """Test that annotations module can be imported."""
        from anncsu.common.utils import annotations

        assert annotations is not None

    def test_import_datetimes(self):
        """Test that datetimes module can be imported."""
        from anncsu.common.utils import datetimes

        assert datetimes is not None

    def test_import_enums(self):
        """Test that enums module can be imported."""
        from anncsu.common.utils import enums

        assert enums is not None

    def test_import_eventstreaming(self):
        """Test that eventstreaming module can be imported."""
        from anncsu.common.utils import eventstreaming

        assert eventstreaming is not None

    def test_import_forms(self):
        """Test that forms module can be imported."""
        from anncsu.common.utils import forms

        assert forms is not None

    def test_import_headers(self):
        """Test that headers module can be imported."""
        from anncsu.common.utils import headers

        assert headers is not None

    def test_import_logger(self):
        """Test that logger module can be imported."""
        from anncsu.common.utils import logger

        assert logger is not None

    def test_import_metadata(self):
        """Test that metadata module can be imported."""
        from anncsu.common.utils import metadata

        assert metadata is not None

    def test_import_queryparams(self):
        """Test that queryparams module can be imported."""
        from anncsu.common.utils import queryparams

        assert queryparams is not None

    def test_import_requestbodies(self):
        """Test that requestbodies module can be imported."""
        from anncsu.common.utils import requestbodies

        assert requestbodies is not None

    def test_import_retries(self):
        """Test that retries module can be imported."""
        from anncsu.common.utils import retries

        assert retries is not None

    def test_import_security(self):
        """Test that security module can be imported."""
        from anncsu.common.utils import security

        assert security is not None

    def test_import_serializers(self):
        """Test that serializers module can be imported."""
        from anncsu.common.utils import serializers

        assert serializers is not None

    def test_import_unmarshal_json_response(self):
        """Test that unmarshal_json_response module can be imported."""
        from anncsu.common.utils import unmarshal_json_response

        assert unmarshal_json_response is not None

    def test_import_url(self):
        """Test that url module can be imported."""
        from anncsu.common.utils import url

        assert url is not None

    def test_import_values(self):
        """Test that values module can be imported."""
        from anncsu.common.utils import values

        assert values is not None


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from anncsu.common.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ElencoOdonimiPostRequestTypedDict(TypedDict):
    r"""search input data"""

    req: NotRequired[str]
    codcom: NotRequired[str]
    denomparz: NotRequired[str]


class ElencoOdonimiPostRequest(BaseModel):
    r"""search input data"""

    req: Optional[str] = None

    codcom: Optional[str] = None

    denomparz: Optional[str] = None


class ElencoOdonimiPostDataTypedDict(TypedDict):
    dug: NotRequired[str]
    denomuff: NotRequired[str]
    denomloc: NotRequired[str]
    denomlingua1: NotRequired[str]
    denomlingua2: NotRequired[str]


class ElencoOdonimiPostData(BaseModel):
    dug: Optional[str] = None

    denomuff: Optional[str] = None

    denomloc: Optional[str] = None

    denomlingua1: Optional[str] = None

    denomlingua2: Optional[str] = None


class ElencoOdonimiPostResponseTypedDict(TypedDict):
    r"""Successful operation"""

    res: NotRequired[str]
    data: NotRequired[List[ElencoOdonimiPostDataTypedDict]]


class ElencoOdonimiPostResponse(BaseModel):
    r"""Successful operation"""

    res: Optional[str] = None

    data: Optional[List[ElencoOdonimiPostData]] = None



"""Property-based tests for validators using Hypothesis and Faker."""

import base64
import string

import pytest
from faker import Faker
from hypothesis import assume, given, settings
from hypothesis import strategies as st

from anncsu.common.validation import base64_validator, belfiore_code_validator

# Initialize Faker
fake = Faker("it_IT")  # Italian locale for realistic Italian data


class TestBase64ValidatorPropertyBased:
    """Property-based tests for base64_validator."""

    @given(st.binary())
    @settings(max_examples=100)
    def test_any_bytes_can_be_base64_encoded(self, data: bytes):
        """Test that any bytes can be encoded to valid base64."""
        encoded = base64.b64encode(data).decode("ascii")
        # Should not raise
        result = base64_validator(encoded)
        assert result == encoded

    @given(st.text(alphabet=string.ascii_letters + string.digits + "+/=", min_size=4))
    @settings(max_examples=100)
    def test_valid_base64_alphabet_accepts(self, text: str):
        """Test that strings with valid base64 alphabet are handled correctly."""
        # Pad to make it valid base64 length
        padded = text + "=" * ((4 - len(text) % 4) % 4)

        try:
            # Try to decode - if it works, validator should accept it
            base64.b64decode(padded, validate=True)
            result = base64_validator(padded)
            assert result == padded
        except Exception:
            # If decode fails, validator should reject it
            with pytest.raises(ValueError, match="Invalid base64"):
                base64_validator(padded)

    @given(
        st.text(
            alphabet=string.printable.replace("+", "")
            .replace("/", "")
            .replace("=", ""),
            min_size=1,
        ).filter(
            lambda x: not all(c in string.ascii_letters + string.digits for c in x)
        )
    )
    @settings(max_examples=50)
    def test_invalid_characters_rejected(self, text: str):
        """Test that strings with invalid base64 characters are rejected."""
        with pytest.raises(ValueError, match="Invalid base64"):
            base64_validator(text)

    @given(st.text(min_size=1, max_size=200))
    @settings(max_examples=100)
    def test_round_trip_encoding(self, text: str):
        """Test that encoding and validating text works correctly."""
        encoded = base64.b64encode(text.encode("utf-8")).decode("ascii")
        result = base64_validator(encoded)
        assert result == encoded

        # Verify we can decode it back
        decoded = base64.b64decode(result).decode("utf-8")
        assert decoded == text


class TestBelfioreCodeValidatorPropertyBased:
    """Property-based tests for belfiore_code_validator."""

    @given(
        st.text(alphabet=string.ascii_uppercase, min_size=1, max_size=1),
        st.integers(min_value=0, max_value=999),
    )
    @settings(max_examples=200)
    def test_valid_belfiore_format(self, letter: str, number: int):
        """Test that valid Belfiore codes are accepted."""
        code = f"{letter}{number:03d}"
        result = belfiore_code_validator(code)
        assert result == code
        assert len(result) == 4
        assert result[0].isupper()
        assert result[1:].isdigit()

    @given(st.text(alphabet=string.ascii_lowercase, min_size=1, max_size=1))
    @settings(max_examples=50)
    def test_lowercase_letter_rejected(self, letter: str):
        """Test that lowercase letters are rejected."""
        code = f"{letter}501"
        with pytest.raises(ValueError, match="Invalid Belfiore code"):
            belfiore_code_validator(code)

    @given(
        st.integers(min_value=0, max_value=9999).filter(lambda x: x < 1000 or x > 9999)
    )
    @settings(max_examples=50)
    def test_wrong_length_rejected(self, number: int):
        """Test that wrong length codes are rejected."""
        if number < 10:
            code = f"A{number}"  # Too short
        elif number >= 10 and number < 100:
            code = f"A{number}"  # Too short
        else:
            code = f"A{number}"  # Too long

        # Only test if not exactly 3 digits after letter
        if len(code) != 4:
            with pytest.raises(ValueError, match="Invalid Belfiore code"):
                belfiore_code_validator(code)

    @given(
        st.text(min_size=4, max_size=4).filter(
            lambda x: not (len(x) == 4 and x[0].isupper() and x[1:].isdigit())
        )
    )
    @settings(max_examples=100)
    def test_invalid_patterns_rejected(self, code: str):
        """Test that invalid patterns are rejected."""
        assume(len(code) == 4)  # Only test 4-char strings
        assume(not (code[0].isupper() and code[1:].isdigit()))  # Invalid pattern

        with pytest.raises(ValueError, match="Invalid Belfiore code"):
            belfiore_code_validator(code)

    @given(st.text(alphabet=string.ascii_uppercase, min_size=2))
    @settings(max_examples=50)
    def test_multiple_letters_rejected(self, letters: str):
        """Test that codes with multiple letters are rejected."""
        code = f"{letters}01"
        with pytest.raises(ValueError, match="Invalid Belfiore code"):
            belfiore_code_validator(code)


class TestValidatorsWithFaker:
    """Tests using Faker to generate realistic Italian data."""

    def test_realistic_italian_street_names_base64(self):
        """Test base64 encoding of realistic Italian street names."""
        for _ in range(50):
            street = fake.street_name()
            encoded = base64.b64encode(street.encode("utf-8")).decode("ascii")

            # Should validate successfully
            result = base64_validator(encoded)
            assert result == encoded

            # Should decode back correctly
            decoded = base64.b64decode(result).decode("utf-8")
            assert decoded == street

    def test_realistic_city_names_base64(self):
        """Test base64 encoding of realistic Italian city names."""
        for _ in range(50):
            city = fake.city()
            encoded = base64.b64encode(city.encode("utf-8")).decode("ascii")

            result = base64_validator(encoded)
            assert result == encoded

    def test_realistic_addresses_base64(self):
        """Test base64 encoding of realistic Italian addresses."""
        for _ in range(30):
            address = fake.address()
            encoded = base64.b64encode(address.encode("utf-8")).decode("ascii")

            result = base64_validator(encoded)
            assert result == encoded

    def test_special_characters_in_italian_text(self):
        """Test base64 encoding with Italian special characters."""
        italian_texts = [
            "Via Torino",
            "Piazza San Marco",
            "Corso Vittorio Emanuele II",
            "Viale della Repubblica",
            "Via Nazionale",
            "Circonvallazione Clodia",
        ]

        for text in italian_texts:
            encoded = base64.b64encode(text.encode("utf-8")).decode("ascii")
            result = base64_validator(encoded)
            assert result == encoded

    def test_known_italian_belfiore_codes(self):
        """Test validation of known real Italian Belfiore codes."""
        real_codes = [
            ("H501", "Roma"),
            ("F205", "Milano"),
            ("A794", "Bologna"),
            ("D612", "Firenze"),
            ("G273", "Napoli"),
            ("L219", "Torino"),
            ("E530", "Genova"),
            ("D969", "Palermo"),
            ("C351", "Bari"),
            ("F839", "Venezia"),
        ]

        for code, _city in real_codes:
            result = belfiore_code_validator(code)
            assert result == code


class TestEdgeCases:
    """Test edge cases discovered through property-based testing."""

    def test_empty_string_base64(self):
        """Test that empty string is valid base64 (encodes to empty)."""
        # Empty string encoded is empty string
        result = base64_validator("")
        assert result == ""

    def test_padding_variations(self):
        """Test base64 strings with different padding."""
        test_cases = [
            "YQ==",  # 2 padding chars
            "YWI=",  # 1 padding char
            "YWJj",  # No padding
        ]

        for encoded in test_cases:
            result = base64_validator(encoded)
            assert result == encoded

    def test_belfiore_boundary_values(self):
        """Test Belfiore codes at boundary values."""
        boundary_codes = [
            "A000",  # Minimum number
            "A001",  # Just above minimum
            "A999",  # Maximum number
            "Z000",  # Different letter at boundary
            "Z999",  # Max letter and number
        ]

        for code in boundary_codes:
            result = belfiore_code_validator(code)
            assert result == code

    def test_unicode_in_base64(self):
        """Test base64 encoding of Unicode characters."""
        unicode_strings = [
            "Café",
            "Müller",
            "日本語",
            "Привет",
            "مرحبا",
            "🏠🚗",  # Emojis
        ]

        for text in unicode_strings:
            encoded = base64.b64encode(text.encode("utf-8")).decode("ascii")
            result = base64_validator(encoded)
            assert result == encoded

            # Verify round-trip
            decoded = base64.b64decode(result).decode("utf-8")
            assert decoded == text


class TestCombinedScenarios:
    """Test realistic combined scenarios."""

    def test_realistic_api_request_data(self):
        """Test validation with realistic API request data."""
        for _ in range(20):
            # Generate realistic Italian street name
            street_name = f"Via {fake.last_name()}"
            encoded_street = base64.b64encode(street_name.encode("utf-8")).decode(
                "ascii"
            )

            # Generate realistic Belfiore code pattern
            letter = fake.random_uppercase_letter()
            number = fake.random_int(min=0, max=999)
            belfiore_code = f"{letter}{number:03d}"

            # Both should validate
            validated_street = base64_validator(encoded_street)
            validated_code = belfiore_code_validator(belfiore_code)

            assert validated_street == encoded_street
            assert validated_code == belfiore_code



"""Comprehensive factory and property-based tests for all ANNCSU PA models.

This test suite creates Polyfactory factories and Hypothesis property-based tests
for all 67 Pydantic models in the ANNCSU PA API.
"""

import base64
import random
import string

import pytest
from faker import Faker
from hypothesis import given
from hypothesis import strategies as st
from polyfactory.factories.pydantic_factory import ModelFactory

# Import all models
from anncsu.pa.models.elencoaccessigetpathparamop import (
    ElencoAccessiGetPathParamData,
    ElencoAccessiGetPathParamRequest,
)
from anncsu.pa.models.elencoaccessiproggetpathparamop import (
    ElencoaccessiprogGetPathParamData,
    ElencoaccessiprogGetPathParamRequest,
)
from anncsu.pa.models.elencoodonimigetpathparamop import (
    ElencoOdonimiGetPathParamData,
    ElencoOdonimiGetPathParamRequest,
    ElencoOdonimiGetPathParamResponse,
)
from anncsu.pa.models.elencoodonimigetqueryparamop import (
    ElencoOdonimiGetQueryParamRequest,
)
from anncsu.pa.models.elencoodonimipostop import (
    ElencoOdonimiPostRequest,
)
from anncsu.pa.models.elencoodonimiproggetpathparamop import (
    ElencoodonimiprogGetPathParamData,
    ElencoodonimiprogGetPathParamRequest,
)
from anncsu.pa.models.esisteaccessogetpathparamop import (
    EsisteAccessoGetPathParamRequest,
)
from anncsu.pa.models.esisteaccessogetqueryparamop import (
    EsisteAccessoGetQueryParamRequest,
)
from anncsu.pa.models.esisteaccessopostop import (
    EsisteAccessoPostRequest,
)
from anncsu.pa.models.esisteodonimogetpathparamop import (
    EsisteOdonimoGetPathParamRequest,
    EsisteOdonimoGetPathParamResponse,
)
from anncsu.pa.models.esisteodonimogetqueryparamop import (
    EsisteOdonimoGetQueryParamRequest,
    EsisteOdonimoGetQueryParamResponse,
)
from anncsu.pa.models.esisteodonimopostop import (
    EsisteOdonimoPostRequest,
    EsisteOdonimoPostResponse,
)
from anncsu.pa.models.prognazaccgetpathparamop import (
    PrognazaccGetPathParamData,
)
from anncsu.pa.models.prognazareagetpathparamop import (
    PrognazareaGetPathParamData,
)
from anncsu.pa.models.show_statusop import ShowStatusResponse

# Initialize Faker with Italian locale
fake = Faker("it_IT")


# Custom providers for Italian data
class BelfioreCodeProvider:
    """Provides valid Italian Belfiore codes."""

    KNOWN_CODES = [
        "H501",  # Roma
        "F205",  # Milano
        "A794",  # Bologna
        "L219",  # Torino
        "D612",  # Firenze
        "C351",  # Catania
        "B157",  # Bari
        "G273",  # Palermo
        "D969",  # Genova
        "F839",  # Napoli
    ]

    @staticmethod
    def belfiore_code() -> str:
        """Generate a valid Belfiore code (letter + 3 digits)."""
        if random.random() < 0.3:  # 30% chance of using known code
            return random.choice(BelfioreCodeProvider.KNOWN_CODES)
        letter = random.choice(string.ascii_uppercase)
        digits = "".join(random.choices(string.digits, k=3))
        return f"{letter}{digits}"


class ItalianStreetProvider:
    """Provides Italian street names and types."""

    STREET_TYPES = [
        "VIA",
        "VIALE",
        "PIAZZA",
        "CORSO",
        "VICOLO",
        "LARGO",
        "STRADA",
        "AUTOSTRADA",
    ]

    @staticmethod
    def street_name() -> str:
        """Generate an Italian street name."""
        return fake.street_name().upper()

    @staticmethod
    def street_name_base64() -> str:
        """Generate a base64-encoded Italian street name."""
        name = ItalianStreetProvider.street_name()
        return base64.b64encode(name.encode()).decode()

    @staticmethod
    def street_type() -> str:
        """Generate an Italian street type (DUG)."""
        return random.choice(ItalianStreetProvider.STREET_TYPES)


# ===========================
# FACTORY DEFINITIONS
# ===========================


# Note: We don't need standalone factories for Belfiore codes or Base64 strings
# These are generated directly by provider classes and used in model factories


# Request Factories - EsisteOdonimo
class EsisteOdonimoGetPathParamRequestFactory(
    ModelFactory[EsisteOdonimoGetPathParamRequest]
):
    __model__ = EsisteOdonimoGetPathParamRequest
    __check_model__ = False

    codcom = BelfioreCodeProvider.belfiore_code
    denom = ItalianStreetProvider.street_name_base64


class EsisteOdonimoGetQueryParamRequestFactory(
    ModelFactory[EsisteOdonimoGetQueryParamRequest]
):
    __model__ = EsisteOdonimoGetQueryParamRequest
    __check_model__ = False

    codcom = BelfioreCodeProvider.belfiore_code
    denom = ItalianStreetProvider.street_name_base64


class EsisteOdonimoPostRequestFactory(ModelFactory[EsisteOdonimoPostRequest]):
    __model__ = EsisteOdonimoPostRequest
    __check_model__ = False


# Response Factories - EsisteOdonimo
class EsisteOdonimoGetPathParamResponseFactory(
    ModelFactory[EsisteOdonimoGetPathParamResponse]
):
    __model__ = EsisteOdonimoGetPathParamResponse
    __check_model__ = False


class EsisteOdonimoGetQueryParamResponseFactory(
    ModelFactory[EsisteOdonimoGetQueryParamResponse]
):
    __model__ = EsisteOdonimoGetQueryParamResponse
    __check_model__ = False


class EsisteOdonimoPostResponseFactory(ModelFactory[EsisteOdonimoPostResponse]):
    __model__ = EsisteOdonimoPostResponse
    __check_model__ = False


# Request Factories - EsisteAccesso
class EsisteAccessoGetPathParamRequestFactory(
    ModelFactory[EsisteAccessoGetPathParamRequest]
):
    __model__ = EsisteAccessoGetPathParamRequest
    __check_model__ = False

    codcom = BelfioreCodeProvider.belfiore_code
    denom = ItalianStreetProvider.street_name_base64


class EsisteAccessoGetQueryParamRequestFactory(
    ModelFactory[EsisteAccessoGetQueryParamRequest]
):
    __model__ = EsisteAccessoGetQueryParamRequest
    __check_model__ = False

    codcom = BelfioreCodeProvider.belfiore_code
    denom = ItalianStreetProvider.street_name_base64


class EsisteAccessoPostRequestFactory(ModelFactory[EsisteAccessoPostRequest]):
    __model__ = EsisteAccessoPostRequest
    __check_model__ = False


# Data Factories - ElencoOdonimi
class ElencoOdonimiDataFactory(ModelFactory[ElencoOdonimiGetPathParamData]):
    __model__ = ElencoOdonimiGetPathParamData
    __check_model__ = False

    dug = ItalianStreetProvider.street_type
    denomuff = ItalianStreetProvider.street_name


# Request Factories - ElencoOdonimi
class ElencoOdonimiGetPathParamRequestFactory(
    ModelFactory[ElencoOdonimiGetPathParamRequest]
):
    __model__ = ElencoOdonimiGetPathParamRequest
    __check_model__ = False

    codcom = BelfioreCodeProvider.belfiore_code
    denomparz = ItalianStreetProvider.street_name_base64


class ElencoOdonimiGetQueryParamRequestFactory(
    ModelFactory[ElencoOdonimiGetQueryParamRequest]
):
    __model__ = ElencoOdonimiGetQueryParamRequest
    __check_model__ = False

    codcom = BelfioreCodeProvider.belfiore_code
    denomparz = ItalianStreetProvider.street_name_base64


class ElencoOdonimiPostRequestFactory(ModelFactory[ElencoOdonimiPostRequest]):
    __model__ = ElencoOdonimiPostRequest
    __check_model__ = False


# Response Factories - ElencoOdonimi
class ElencoOdonimiGetPathParamResponseFactory(
    ModelFactory[ElencoOdonimiGetPathParamResponse]
):
    __model__ = ElencoOdonimiGetPathParamResponse
    __check_model__ = False


# Data Factories - ElencoodonimProg
class ElencoodonimiprogDataFactory(ModelFactory[ElencoodonimiprogGetPathParamData]):
    __model__ = ElencoodonimiprogGetPathParamData
    __check_model__ = False

    dug = ItalianStreetProvider.street_type
    denomuff = ItalianStreetProvider.street_name


# Request Factories - ElencoodonimProg
class ElencoodonimiprogGetPathParamRequestFactory(
    ModelFactory[ElencoodonimiprogGetPathParamRequest]
):
    __model__ = ElencoodonimiprogGetPathParamRequest
    __check_model__ = False

    codcom = BelfioreCodeProvider.belfiore_code
    denomparz = ItalianStreetProvider.street_name_base64


# Data Factories - ElencoAccessi
class ElencoAccessiDataFactory(ModelFactory[ElencoAccessiGetPathParamData]):
    __model__ = ElencoAccessiGetPathParamData
    __check_model__ = False


# Request Factories - ElencoAccessi
class ElencoAccessiGetPathParamRequestFactory(
    ModelFactory[ElencoAccessiGetPathParamRequest]
):
    __model__ = ElencoAccessiGetPathParamRequest
    __check_model__ = False

    codcom = BelfioreCodeProvider.belfiore_code
    denom = ItalianStreetProvider.street_name_base64


# Data Factories - ElencoaccessiProg
class ElencoaccessiprogDataFactory(ModelFactory[ElencoaccessiprogGetPathParamData]):
    __model__ = ElencoaccessiprogGetPathParamData
    __check_model__ = False


# Request Factories - ElencoaccessiProg
class ElencoaccessiprogGetPathParamRequestFactory(
    ModelFactory[ElencoaccessiprogGetPathParamRequest]
):
    __model__ = ElencoaccessiprogGetPathParamRequest
    __check_model__ = False


# Data Factories - Prognazarea
class PrognazareaDataFactory(ModelFactory[PrognazareaGetPathParamData]):
    __model__ = PrognazareaGetPathParamData
    __check_model__ = False

    dug = ItalianStreetProvider.street_type
    denomuff = ItalianStreetProvider.street_name


# Data Factories - Prognazacc
class PrognazaccDataFactory(ModelFactory[PrognazaccGetPathParamData]):
    __model__ = PrognazaccGetPathParamData
    __check_model__ = False

    dug = ItalianStreetProvider.street_type
    denomuff = ItalianStreetProvider.street_name


# Status Response Factory
class ShowStatusResponseFactory(ModelFactory[ShowStatusResponse]):
    __model__ = ShowStatusResponse
    __check_model__ = False


# ===========================
# FACTORY TESTS
# ===========================


class TestEsisteOdonimoFactories:
    """Test factories for EsisteOdonimo operations."""

    def test_esiste_odonimo_get_path_param_request_factory(self):
        """Test factory generates valid EsisteOdonimo GetPathParam requests."""
        for _ in range(10):
            request = EsisteOdonimoGetPathParamRequestFactory.build()
            assert request.codcom is not None
            assert len(request.codcom) == 4
            assert request.codcom[0].isupper()
            assert request.codcom[1:].isdigit()
            assert request.denom is not None
            # Verify it's valid base64
            try:
                base64.b64decode(request.denom)
            except Exception:
                pytest.fail("Invalid base64 in denom field")

    def test_esiste_odonimo_get_query_param_request_factory(self):
        """Test factory generates valid EsisteOdonimo GetQueryParam requests."""
        request = EsisteOdonimoGetQueryParamRequestFactory.build()
        assert request.codcom is not None
        assert request.denom is not None

    def test_esiste_odonimo_post_request_factory(self):
        """Test factory generates valid EsisteOdonimo POST requests."""
        request = EsisteOdonimoPostRequestFactory.build()
        # POST fields are optional
        assert request is not None

    def test_esiste_odonimo_response_factories(self):
        """Test factories generate valid EsisteOdonimo responses."""
        response1 = EsisteOdonimoGetPathParamResponseFactory.build()
        response2 = EsisteOdonimoGetQueryParamResponseFactory.build()
        response3 = EsisteOdonimoPostResponseFactory.build()

        for response in [response1, response2, response3]:
            assert response is not None
            # data field can be bool or None
            if response.data is not None:
                assert isinstance(response.data, bool)

    def test_esiste_odonimo_batch_generation(self):
        """Test batch generation of EsisteOdonimo requests."""
        requests = EsisteOdonimoGetPathParamRequestFactory.batch(50)
        assert len(requests) == 50
        # Check for variety in generated data
        codcoms = {r.codcom for r in requests}
        assert len(codcoms) > 5  # At least some variety


class TestEsisteAccessoFactories:
    """Test factories for EsisteAccesso operations."""

    def test_esiste_accesso_get_path_param_request_factory(self):
        """Test factory generates valid EsisteAccesso GetPathParam requests."""
        request = EsisteAccessoGetPathParamRequestFactory.build()
        assert request.codcom is not None
        assert request.denom is not None
        assert request.accesso is not None

    def test_esiste_accesso_get_query_param_request_factory(self):
        """Test factory generates valid EsisteAccesso GetQueryParam requests."""
        request = EsisteAccessoGetQueryParamRequestFactory.build()
        assert request.codcom is not None
        assert request.denom is not None
        assert request.accesso is not None

    def test_esiste_accesso_post_request_factory(self):
        """Test factory generates valid EsisteAccesso POST requests."""
        request = EsisteAccessoPostRequestFactory.build()
        assert request is not None


class TestElencoOdonimiFactories:
    """Test factories for ElencoOdonimi operations."""

    def test_elenco_odonimi_data_factory(self):
        """Test factory generates valid ElencoOdonimi data models."""
        for _ in range(10):
            data = ElencoOdonimiDataFactory.build()
            assert data is not None
            if data.dug:
                assert data.dug in ItalianStreetProvider.STREET_TYPES
            if data.denomuff:
                assert isinstance(data.denomuff, str)

    def test_elenco_odonimi_get_path_param_request_factory(self):
        """Test factory generates valid ElencoOdonimi GetPathParam requests."""
        request = ElencoOdonimiGetPathParamRequestFactory.build()
        assert request.codcom is not None
        assert len(request.codcom) == 4
        assert request.denomparz is not None
        # Verify base64
        try:
            base64.b64decode(request.denomparz)
        except Exception:
            pytest.fail("Invalid base64 in denomparz field")

    def test_elenco_odonimi_get_query_param_request_factory(self):
        """Test factory generates valid ElencoOdonimi GetQueryParam requests."""
        request = ElencoOdonimiGetQueryParamRequestFactory.build()
        assert request.codcom is not None
        assert request.denomparz is not None

    def test_elenco_odonimi_response_factory(self):
        """Test factory generates valid ElencoOdonimi responses."""
        response = ElencoOdonimiGetPathParamResponseFactory.build()
        assert response is not None
        if response.data is not None:
            assert isinstance(response.data, list)


class TestElencoodonimiprogFactories:
    """Test factories for Elencoodonimiprog operations."""

    def test_elencoodonimiprog_data_factory(self):
        """Test factory generates valid Elencoodonimiprog data models."""
        data = ElencoodonimiprogDataFactory.build()
        assert data is not None
        # prognaz field should be present (optional but typically set)
        if data.prognaz:
            assert isinstance(data.prognaz, str)

    def test_elencoodonimiprog_request_factory(self):
        """Test factory generates valid Elencoodonimiprog requests."""
        request = ElencoodonimiprogGetPathParamRequestFactory.build()
        assert request.codcom is not None
        assert request.denomparz is not None


class TestElencoAccessiFactories:
    """Test factories for ElencoAccessi operations."""

    def test_elenco_accessi_data_factory(self):
        """Test factory generates valid ElencoAccessi data models."""
        data = ElencoAccessiDataFactory.build()
        assert data is not None
        # Check optional fields exist
        if data.civico:
            assert isinstance(data.civico, str)

    def test_elenco_accessi_request_factory(self):
        """Test factory generates valid ElencoAccessi requests."""
        request = ElencoAccessiGetPathParamRequestFactory.build()
        assert request.codcom is not None
        assert request.denom is not None
        assert request.accparz is not None


class TestElencoaccessiprogFactories:
    """Test factories for Elencoaccessiprog operations."""

    def test_elencoaccessiprog_data_factory(self):
        """Test factory generates valid Elencoaccessiprog data models."""
        data = ElencoaccessiprogDataFactory.build()
        assert data is not None
        # Check for coordinate fields
        if data.coord_x:
            assert isinstance(data.coord_x, str)
        if data.coord_y:
            assert isinstance(data.coord_y, str)

    def test_elencoaccessiprog_request_factory(self):
        """Test factory generates valid Elencoaccessiprog requests."""
        request = ElencoaccessiprogGetPathParamRequestFactory.build()
        assert request.prognaz is not None
        assert request.accparz is not None


class TestPrognazareaFactories:
    """Test factories for Prognazarea operations."""

    def test_prognazarea_data_factory(self):
        """Test factory generates valid Prognazarea data models."""
        data = PrognazareaDataFactory.build()
        assert data is not None
        if data.prognaz:
            assert isinstance(data.prognaz, str)


class TestPrognazaccFactories:
    """Test factories for Prognazacc operations."""

    def test_prognazacc_data_factory(self):
        """Test factory generates valid Prognazacc data models."""
        data = PrognazaccDataFactory.build()
        assert data is not None
        # Prognazacc has both street and access data
        if data.prognaz:
            assert isinstance(data.prognaz, str)
        if data.prognazacc:
            assert isinstance(data.prognazacc, str)


class TestShowStatusFactory:
    """Test factory for ShowStatus operation."""

    def test_show_status_response_factory(self):
        """Test factory generates valid ShowStatus responses."""
        response = ShowStatusResponseFactory.build()
        assert response is not None


# ===========================
# PROPERTY-BASED TESTS (HYPOTHESIS)
# ===========================


@st.composite
def belfiore_code_strategy(draw):
    """Hypothesis strategy for generating Belfiore codes."""
    letter = draw(st.sampled_from(string.ascii_uppercase))
    digits = draw(st.text(alphabet=string.digits, min_size=3, max_size=3))
    return f"{letter}{digits}"


@st.composite
def base64_street_name_strategy(draw):
    """Hypothesis strategy for generating base64-encoded street names."""
    # Generate Italian-ish street name
    name = draw(st.text(alphabet=string.ascii_uppercase + " ", min_size=5, max_size=50))
    return base64.b64encode(name.encode()).decode()


class TestPropertyBasedValidation:
    """Property-based tests using Hypothesis."""

    @given(belfiore_code_strategy())
    def test_belfiore_code_format(self, codcom):
        """Test Belfiore code format with Hypothesis."""
        assert len(codcom) == 4
        assert codcom[0].isupper()
        assert codcom[1:].isdigit()

    @given(base64_street_name_strategy())
    def test_base64_encoding_valid(self, denom):
        """Test base64 encoding is valid."""
        try:
            decoded = base64.b64decode(denom)
            assert isinstance(decoded, bytes)
        except Exception:
            pytest.fail(f"Invalid base64: {denom}")

    @given(codcom=belfiore_code_strategy(), denom=base64_street_name_strategy())
    def test_esiste_odonimo_request_with_hypothesis(self, codcom, denom):
        """Test EsisteOdonimoRequest creation with Hypothesis-generated data."""
        request = EsisteOdonimoGetPathParamRequest(codcom=codcom, denom=denom)
        assert request.codcom == codcom
        assert request.denom == denom

    @given(st.booleans())
    def test_esiste_odonimo_response_with_hypothesis(self, data_value):
        """Test EsisteOdonimoResponse with Hypothesis-generated boolean."""
        response = EsisteOdonimoGetPathParamResponse(
            res="esisteodonimo", data=data_value
        )
        assert response.data == data_value
        assert isinstance(response.data, bool)


class TestFactoryIntegrationWithModels:
    """Test factories work correctly with actual model validation."""

    @pytest.mark.parametrize(
        "factory_class",
        [
            EsisteOdonimoGetPathParamRequestFactory,
            EsisteOdonimoGetQueryParamRequestFactory,
            EsisteAccessoGetPathParamRequestFactory,
            EsisteAccessoGetQueryParamRequestFactory,
            ElencoOdonimiGetPathParamRequestFactory,
            ElencoOdonimiGetQueryParamRequestFactory,
            ElencoodonimiprogGetPathParamRequestFactory,
            ElencoAccessiGetPathParamRequestFactory,
            ElencoaccessiprogGetPathParamRequestFactory,
        ],
    )
    def test_request_factories_produce_valid_models(self, factory_class):
        """Test that all request factories produce valid Pydantic models."""
        for _ in range(5):
            model = factory_class.build()
            # Pydantic will validate on construction
            assert model is not None
            # Verify we can serialize to dict (Pydantic validation)
            model_dict = model.model_dump()
            assert isinstance(model_dict, dict)

    @pytest.mark.parametrize(
        "data_factory_class",
        [
            ElencoOdonimiDataFactory,
            ElencoodonimiprogDataFactory,
            ElencoAccessiDataFactory,
            ElencoaccessiprogDataFactory,
            PrognazareaDataFactory,
            PrognazaccDataFactory,
        ],
    )
    def test_data_factories_produce_valid_models(self, data_factory_class):
        """Test that all data factories produce valid Pydantic models."""
        for _ in range(5):
            model = data_factory_class.build()
            assert model is not None
            # Verify we can serialize
            model_dict = model.model_dump()
            assert isinstance(model_dict, dict)


class TestBatchGeneration:
    """Test batch generation capabilities of factories."""

    def test_batch_generation_produces_variety(self):
        """Test that batch generation produces variety in data."""
        batch = EsisteOdonimoGetPathParamRequestFactory.batch(100)
        assert len(batch) == 100

        # Check for variety in codcom
        codcoms = {req.codcom for req in batch}
        assert len(codcoms) > 10  # Should have variety

        # Check all are valid
        for req in batch:
            assert len(req.codcom) == 4
            assert req.codcom[0].isupper()

    def test_batch_generation_elenco_odonimi_data(self):
        """Test batch generation of ElencoOdonimi data models."""
        batch = ElencoOdonimiDataFactory.batch(50)
        assert len(batch) == 50

        # Check for variety in street types
        dugs = {data.dug for data in batch if data.dug}
        assert len(dugs) > 1  # Should have variety in street types


class TestRealWorldScenarios:
    """Test factories in realistic usage scenarios."""

    def test_create_elenco_odonimi_response_with_multiple_items(self):
        """Test creating a response with multiple data items."""
        data_items = ElencoOdonimiDataFactory.batch(5)
        response = ElencoOdonimiGetPathParamResponse(
            res="elencoodonimi", data=data_items
        )

        assert response.res == "elencoodonimi"
        assert len(response.data) == 5
        for item in response.data:
            assert isinstance(item, ElencoOdonimiGetPathParamData)

    def test_esiste_odonimo_true_false_scenarios(self):
        """Test both true and false scenarios for esiste operations."""
        response_true = EsisteOdonimoGetPathParamResponse(
            res="esisteodonimo", data=True
        )
        response_false = EsisteOdonimoGetPathParamResponse(
            res="esisteodonimo", data=False
        )

        assert response_true.data is True
        assert response_false.data is False

    def test_coordinates_in_elencoaccessiprog_data(self):
        """Test coordinate fields in Elencoaccessiprog data."""
        data = ElencoaccessiprogDataFactory.build(
            coord_x="12.4963655", coord_y="41.9027835", quota="50", metodo="4"
        )

        assert data.coord_x == "12.4963655"
        assert data.coord_y == "41.9027835"
        assert data.quota == "50"
        assert data.metodo == "4"

    def test_nullable_fields_in_data_models(self):
        """Test that nullable fields can be None."""
        data = ElencoOdonimiDataFactory.build(
            dug="VIA",
            denomuff="ROMA",
            denomloc=None,
            denomlingua1=None,
            denomlingua2=None,
        )

        assert data.dug == "VIA"
        assert data.denomuff == "ROMA"
        assert data.denomloc is None
        assert data.denomlingua1 is None
        assert data.denomlingua2 is None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from anncsu.common.errors import AnncsuBaseError as AnncsuError
from anncsu.common.types import BaseModel
import httpx
from typing import Optional


class EsisteOdonimoPostInternalServerErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class EsisteOdonimoPostInternalServerError(AnncsuError):
    r"""Internal Server Error"""

    data: EsisteOdonimoPostInternalServerErrorData

    def __init__(
        self,
        data: EsisteOdonimoPostInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class EsisteOdonimoPostUnprocessableEntityErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class EsisteOdonimoPostUnprocessableEntityError(AnncsuError):
    r"""Unprocessable Entity - error in json"""

    data: EsisteOdonimoPostUnprocessableEntityErrorData

    def __init__(
        self,
        data: EsisteOdonimoPostUnprocessableEntityErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class EsisteOdonimoPostMethodNotAllowedErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class EsisteOdonimoPostMethodNotAllowedError(AnncsuError):
    r"""Method Not Allowed"""

    data: EsisteOdonimoPostMethodNotAllowedErrorData

    def __init__(
        self,
        data: EsisteOdonimoPostMethodNotAllowedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class EsisteOdonimoPostBadRequestErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class EsisteOdonimoPostBadRequestError(AnncsuError):
    r"""Bad Request"""

    data: EsisteOdonimoPostBadRequestErrorData

    def __init__(
        self,
        data: EsisteOdonimoPostBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from anncsu.common.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ElencoAccessiPostRequestTypedDict(TypedDict):
    r"""search input data"""

    req: NotRequired[str]
    codcom: NotRequired[str]
    denom: NotRequired[str]
    accparz: NotRequired[str]


class ElencoAccessiPostRequest(BaseModel):
    r"""search input data"""

    req: Optional[str] = None

    codcom: Optional[str] = None

    denom: Optional[str] = None

    accparz: Optional[str] = None


class ElencoAccessiPostDataTypedDict(TypedDict):
    civico: NotRequired[str]
    esp: NotRequired[str]
    specif: NotRequired[str]
    metrico: NotRequired[str]


class ElencoAccessiPostData(BaseModel):
    civico: Optional[str] = None

    esp: Optional[str] = None

    specif: Optional[str] = None

    metrico: Optional[str] = None


class ElencoAccessiPostResponseTypedDict(TypedDict):
    r"""Successful operation"""

    res: NotRequired[str]
    data: NotRequired[List[ElencoAccessiPostDataTypedDict]]


class ElencoAccessiPostResponse(BaseModel):
    r"""Successful operation"""

    res: Optional[str] = None

    data: Optional[List[ElencoAccessiPostData]] = None
